# -*- encoding: utf-8 -*-
import os
import textwrap
from abjad.tools import documentationtools
from abjad.tools.abctools.AbjadObject import AbjadObject


class CodeBlock(AbjadObject):
    r'''A code block.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_hide',
        '_lines',
        '_processed_results',
        '_scale',
        '_starting_line_number',
        '_stopping_line_number',
        '_strip_prompt',
        '_wrap_width',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        hide=False,
        lines=None,
        scale=None,
        starting_line_number=0,
        stopping_line_number=1,
        strip_prompt=False,
        wrap_width=None
        ):
        lines = lines or []
        assert starting_line_number <= stopping_line_number
        self._hide = bool(hide)
        self._lines = tuple(lines)
        self._processed_results = None
        self._scale = scale
        self._starting_line_number = starting_line_number
        self._stopping_line_number = stopping_line_number
        self._strip_prompt = bool(strip_prompt)
        self._wrap_width = wrap_width

    ### SPECIAL METHODS ###

    def __call__(
        self,
        processor,
        pipe,
        image_count=0,
        directory=None,
        image_prefix='image',
        verbose=False,
        ):
        r'''Calls code block.

        Returns image count.
        '''

        assert isinstance(pipe, documentationtools.Pipe)

        if verbose:
            print('\nCODEBLOCK: {}:{}'.format(
                self.starting_line_number,
                self.stopping_line_number
                ))

        grouped_results = []
        result = []

        pipe.write('\n')

        previous_line_was_empty = False

        for line in self.lines:

            processor.update_status(line)

            hide = self.hide

            if not self.strip_prompt or not previous_line_was_empty:
                current = self.read(pipe)

            if line.endswith('<hide'):
                hide = True
                line = line.rpartition('<hide')[0]

            page_range = None
            if '<page' in line:
                parts = line.rpartition('<page')
                line = parts[0]
                page_range = parts[-1].strip()
                if '-' in page_range:
                    parts = page_range.partition('-')
                    page_range = range(int(parts[0]), int(parts[-1]) + 1)
                else:
                    page_range = [int(page_range)]

            if not hide:
                current[-1] += line
                result.extend(current)

            if verbose:
                for x in current:
                    print(x)

            if line.startswith('show('):
                image_count += 1
                file_name = '{}-{}'.format(image_prefix, image_count)
                if ',' in line:
                    object_name = line.split(',')[0][5:].strip()
                else:
                    object_name = line.partition(')')[0][5:]
                file_path = file_name + '.ly'
                if directory:
                    file_path = os.path.join(directory, file_path)
                command = '__result__ = persist(documentationtools.make_reference_manual_lilypond_file({})).as_ly({!r})'.format(
                    object_name, file_path)
                pipe.write(command)
                grouped_results.append(result)
                image_dict = {
                    'file_name': file_name,
                    'image_count': image_count,
                    'image_prefix': image_prefix,
                    'page_range': page_range,
                    'scale': self.scale,
                    }
                grouped_results.append(image_dict)
                result = []
                pipe.write('\n')
                previous_line_was_empty = False

            else:
                if not self.strip_prompt or len(line.strip()):
                    pipe.write(line)
                    pipe.write('\n')
                    previous_line_was_empty = False
                else:
                    previous_line_was_empty = True
                #print '  LINE?', line
                #if len(line.strip()):
                #    #print '  PIPE:', line
                #    pipe.write(line)
                #    pipe.write('\n')
                #    previous_line_was_empty = False
                #else:
                #    previous_line_was_empty = True

        result.extend(self.read(pipe))
        if result[-1] == '>>> ':
            result.pop()

        pipe.write('\n')
        result.extend(self.read(pipe))
        if result[-1] == '>>> ':
            result.pop()

        grouped_results.append(result)

        if self.strip_prompt:
            for result in [group for group in grouped_results
                if isinstance(group, list)]:
                for i, line in enumerate(result):
                    if line.startswith(('>>> ', '... ')):
                        result[i] = line[4:]
                while not result[-1]:
                    result.pop()

        for i, result in enumerate(grouped_results):
            if isinstance(result, list):
                grouped_results[i] = tuple(result)

        self._processed_results = tuple([x for x in grouped_results if x])

        return image_count

    def __eq__(self, expr):
        r'''Is true when `expr` is a code block with lines, starting line
        number, ending line number, hide and strip prompt boolean equal to
        those of this code block. Otherwise false.

        Returns boolean.
        '''
        if type(self) == type(expr) and \
            self.lines == expr.lines and \
            self.starting_line_number == expr.starting_line_number and \
            self.stopping_line_number == expr.stopping_line_number and \
            self.hide == expr.hide and \
            self.strip_prompt == expr.strip_prompt:
            return True
        return False

    def __hash__(self):
        r'''Hashes code block.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(CodeBlock, self).__hash__()

    ### PUBLIC PROPERTIES ###

    @property
    def hide(self):
        r'''Is true when code block should hide.
        '''
        return self._hide

    @property
    def lines(self):
        r'''Lines of code block.
        '''
        return self._lines

    @property
    def processed_results(self):
        r'''Processed results of code block.
        '''
        return self._processed_results

    @property
    def scale(self):
        r'''Image scaling factor.
        '''
        return self._scale

    @property
    def starting_line_number(self):
        r'''Starting line number of code block.
        '''
        return self._starting_line_number

    @property
    def stopping_line_number(self):
        r'''Ending line number of code block.
        '''
        return self._stopping_line_number

    @property
    def strip_prompt(self):
        r'''Is true when code block should strip prompt.
        '''
        return self._strip_prompt

    @property
    def wrap_width(self):
        r'''Wrap width.
        '''
        return self._wrap_width

    ### PUBLIC METHODS ###

    def read(self, pipe):
        r'''Reads `pipe`.
        '''
        # Guarantee we make it to the next prompt.
        # Exceptions sometimes take longer than expected.
        result = pipe.read_wait().replace('\t', '    ').split('\n')
        if result[-1] == '':
            result.pop()
        while result[-1] not in ('>>> ', '... '):
            new = pipe.read_wait().replace('\t', '    ').split('\n')
            if new[-1] == '':
                new.pop()
            result.extend(new)
        if self.wrap_width is not None:
            new_result = []
            for line in result:
                if self.wrap_width < len(line):
                    new_result.extend(textwrap.wrap(line, self.wrap_width))
                else:
                    new_result.append(line)
            return new_result
        return result


# -*- encoding: utf-8 -*-
from abjad.tools import scoretools


def make_basic_lilypond_file(music=None):
    r'''Makes basic LilyPond file.

    ..  container:: example


        ::

            >>> score = Score([Staff("c'8 d'8 e'8 f'8")])
            >>> lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
            >>> lilypond_file.header_block.title = Markup('Missa sexti tonus')
            >>> lilypond_file.header_block.composer = Markup('Josquin')
            >>> lilypond_file.layout_block.indent = 0
            >>> lilypond_file.paper_block.top_margin = 15
            >>> lilypond_file.paper_block.left_margin = 15

        ::

            >>> print(format(lilypond_file)) # doctest: +SKIP
            \header {
                composer = \markup { Josquin }
                title = \markup { Missa sexti tonus }
            }

            \layout {
                indent = #0
            }

            \paper {
                left-margin = #15
                top-margin = #15
            }

            \score {
                \new Score <<
                    \new Staff {
                        c'8
                        d'8
                        e'8
                        f'8
                    }
                >>
            }

        ::

            >>> show(lilypond_file) # doctest: +SKIP

    Wraps `music` in LilyPond ``\score`` block.

    Adds LilyPond ``\header``, ``\layout``, ``\paper`` and ``\score`` blocks to
    LilyPond file.

    Returns LilyPond file.
    '''
    from abjad.tools import lilypondfiletools
    if isinstance(music, lilypondfiletools.LilyPondFile):
        return music
    lilypond_file = lilypondfiletools.LilyPondFile()
    header_block = lilypondfiletools.Block(name='header')
    layout_block = lilypondfiletools.Block(name='layout')
    paper_block = lilypondfiletools.Block(name='paper')
    score_block = lilypondfiletools.Block(name='score')
    lilypond_file.items.extend([
        header_block,
        layout_block,
        paper_block,
        score_block,
        ])
    lilypond_file.header_block = header_block
    lilypond_file.layout_block = layout_block
    lilypond_file.paper_block = paper_block
    lilypond_file.score_block = score_block
    if music is not None:
        score_block.items.append(music)
    return lilypond_file



# -*- encoding: utf-8 -*-
from abjad import *


def test_rhythmtreetools_RhythmTreeContainer___call___01():

    rtm = '(1 (1 (2 (1 1 1)) 2))'
    tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
    result = tree((1, 4))

    assert isinstance(result, (list, selectiontools.SliceSelection))
    assert len(result) == 1

    assert systemtools.TestManager.compare(
        result[0],
        r'''
        \times 4/5 {
            c'16
            \times 2/3 {
                c'16
                c'16
                c'16
            }
            c'8
        }
        '''
        )


def test_rhythmtreetools_RhythmTreeContainer___call___02():

    rtm = '(1 (1 (2 (1 1 1 1)) 1))'
    tree = rhythmtreetools.RhythmTreeParser()(rtm)[0]
    result = tree((1, 4))

    assert isinstance(result, (list, selectiontools.SliceSelection))
    assert len(result) == 6
    assert [format(x) for x in result] == ["c'16", "c'32", "c'32", "c'32", "c'32", "c'16"]


# -*- encoding: utf-8 -*-
from abjad.tools import stringtools
from abjad.tools.abctools import AbjadObject


class Scheme(AbjadObject):
    r'''Abjad model of Scheme code.

    ::

        >>> scheme = schemetools.Scheme(True)
        >>> format(scheme)
        '##t'

    Scheme can represent nested structures:

    ::

        >>> scheme = schemetools.Scheme(
        ...     ('left', (1, 2, False)), ('right', (1, 2, 3.3)))
        >>> format(scheme)
        '#((left (1 2 #f)) (right (1 2 3.3)))'

    Scheme wraps variable-length arguments into a tuple:

    ::

        >>> scheme_1 = schemetools.Scheme(1, 2, 3)
        >>> scheme_2 = schemetools.Scheme((1, 2, 3))
        >>> format(scheme_1) == format(scheme_2)
        True

    Scheme also takes an optional `quoting` keyword,
    by which Scheme's various quote, unquote, unquote-splicing characters
    can be prepended to the formatted result:

    ::

        >>> scheme = schemetools.Scheme((1, 2, 3), quoting="'#")
        >>> format(scheme)
        "#'#(1 2 3)"

    Scheme can also force quotes around strings which contain no whitespace:

    ::

        >>> scheme = schemetools.Scheme('nospaces', force_quotes=True)
        >>> print(format(scheme))
        #"nospaces"

    The above is useful in certain \override situations,
    as LilyPond's Scheme interpreter
    will treat unquoted strings as symbols rather than strings.

    Scheme is immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_force_quotes',
        '_quoting',
        '_value',
        )

    ### INITIALIZER ###

    def __init__(self, *args, **kwargs):
        if 1 == len(args):
            if isinstance(args[0], type(self)):
                args = args[0]._value
            else:
                args = args[0]
        quoting = kwargs.get('quoting')
        force_quotes = bool(kwargs.get('force_quotes'))
        assert isinstance(quoting, (str, type(None)))
        if quoting is not None:
            assert all(x in ("'", ',', '@', '`', '#') for x in quoting)
        self._force_quotes = force_quotes
        self._quoting = quoting
        self._value = args

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true when `expr` is a scheme object with a value equal to that
        of this scheme object. Otherwise false.

        Returns boolean.
        '''
        if type(self) == type(expr):
            if self._value == expr._value:
                return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats scheme.

        Set `format_specification` to `''`', `'lilypond'` or ``'storage'``.
        Interprets `''` equal to `'lilypond'`.

        ::

            >>> scheme = schemetools.Scheme('foo')
            >>> format(scheme)
            '#foo'

        ::

            >>> print(format(scheme, 'storage'))
            schemetools.Scheme(
                'foo'
                )

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'lilypond'):
            return self._lilypond_format
        elif format_specification == 'storage':
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns tuple.
        '''
        return (self._value,)

    def __hash__(self):
        r'''Hashes scheme.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(Scheme, self).__hash__()

    def __str__(self):
        r'''String representation of scheme object.

        Returns string.
        '''
        if self._quoting is not None:
            return self._quoting + self._formatted_value
        return self._formatted_value

    ### PRIVATE PROPERTIES ###

    @property
    def _formatted_value(self):
        from abjad.tools import schemetools
        return schemetools.Scheme.format_scheme_value(
            self._value, force_quotes=self.force_quotes)

    @property
    def _lilypond_format(self):
        if self._quoting is not None:
            return '#' + self._quoting + self._formatted_value
        return '#%s' % self._formatted_value

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        if stringtools.is_string(self._value):
            positional_argument_values = (self._value,)
        else:
            positional_argument_values = self._value
        return systemtools.StorageFormatSpecification(
            self,
            positional_argument_values=positional_argument_values,
            )

    ### PUBLIC METHODS ###

    @staticmethod
    def format_embedded_scheme_value(value, force_quotes=False):
        r'''Formats `value` as an embedded Scheme value.
        '''
        from abjad.tools import schemetools
        result = Scheme.format_scheme_value(value, force_quotes=force_quotes)
        if isinstance(value, bool):
            result = '#{}'.format(result)
        elif isinstance(value, str) and not force_quotes:
            result = '#{}'.format(result)
        elif isinstance(value, schemetools.Scheme):
            result = '#{}'.format(result)
        return result

    @staticmethod
    def format_scheme_value(value, force_quotes=False):
        r'''Formats `value` as Scheme would.

        ::

            >>> schemetools.Scheme.format_scheme_value(1)
            '1'

        ::

            >>> schemetools.Scheme.format_scheme_value('foo')
            'foo'

        ::

            >>> schemetools.Scheme.format_scheme_value('bar baz')
            '"bar baz"'

        ::

            >>> schemetools.Scheme.format_scheme_value([1.5, True, False])
            '(1.5 #t #f)'

        Strings without whitespace can be forcibly quoted via the
        `force_quotes` keyword:

        ::

            >>> schemetools.Scheme.format_scheme_value(
            ...     'foo', force_quotes=True)
            '"foo"'

        Returns string.
        '''
        from abjad.tools import schemetools
        if isinstance(value, str):
            value = value.replace('"', r'\"')
            if -1 == value.find(' ') and not force_quotes:
                return value
            return '"{}"'.format(value)
        elif isinstance(value, bool):
            if value:
                return '#t'
            return '#f'
        elif isinstance(value, (list, tuple)):
            return '({})'.format(
                ' '.join(schemetools.Scheme.format_scheme_value(x)
                    for x in value))
        elif isinstance(value, schemetools.Scheme):
            return str(value)
        elif isinstance(value, type(None)):
            return '#f'
        return str(value)

    ### PUBLIC PROPERTIES ###

    @property
    def force_quotes(self):
        r'''Is true when quotes should be forced in output. Otherwise false.

        Returns boolean.
        '''
        return self._force_quotes



# -*- encoding: utf-8 -*-

# warn on an outdated Python installation
import distutils.version
import platform
if not (
    distutils.version.LooseVersion('2.7.2') <
    distutils.version.LooseVersion(platform.python_version())
    ):
    print('WARNING: Abjad no longer supports' + \
        ' versions of Python less than 2.7.3!')
    print('WARNING: Please upgrade your' + \
        ' version of Python to 2.7.3 or higher!')
del platform
del distutils

# set up tab completion
try:
    import readline
    import rlcompleter
    if readline.__doc__ is not None and 'libedit' in readline.__doc__:
        readline.parse_and_bind("bind ^I rl_complete")
    else:
        readline.parse_and_bind("tab: complete")
    del readline
    del rlcompleter
except ImportError:
    pass

# ensure that the ~/.abjad directory and friends are setup,
# and instantiate Abjad's configuration singleton
from abjad.tools.systemtools.AbjadConfiguration import AbjadConfiguration
abjad_configuration = AbjadConfiguration()
del AbjadConfiguration

# import all tools packages
from abjad.tools import *

# import some frequently used classes for direct user access
from abjad.tools.durationtools import Duration
from abjad.tools.durationtools import Multiplier
from abjad.tools.durationtools import Offset
from abjad.tools.indicatortools import Articulation
from abjad.tools.indicatortools import Clef
from abjad.tools.indicatortools import Dynamic
from abjad.tools.indicatortools import KeySignature
from abjad.tools.indicatortools import Tempo
from abjad.tools.indicatortools import TimeSignature
from abjad.tools.markuptools import Markup
from abjad.tools.pitchtools import NamedPitch
from abjad.tools.scoretools import Chord
from abjad.tools.scoretools import Container
from abjad.tools.scoretools import Measure
from abjad.tools.scoretools import Note
from abjad.tools.scoretools import Rest
from abjad.tools.scoretools import Score
from abjad.tools.scoretools import Staff
from abjad.tools.scoretools import StaffGroup
from abjad.tools.scoretools import Tuplet
from abjad.tools.scoretools import Voice
from abjad.tools.sequencetools import Sequence
from abjad.tools.spannertools import Beam
from abjad.tools.spannertools import Crescendo
from abjad.tools.spannertools import Decrescendo
from abjad.tools.spannertools import Glissando
from abjad.tools.spannertools import Hairpin
from abjad.tools.spannertools import Slur
from abjad.tools.spannertools import Tie
from fractions import Fraction

# import some frequently used functions for direct user access
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import mutate
from abjad.tools.topleveltools import new
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import parse
from abjad.tools.topleveltools import play
from abjad.tools.topleveltools import persist
from abjad.tools.topleveltools import select
from abjad.tools.topleveltools import set_
from abjad.tools.topleveltools import show

# import custom exceptions into the builtins module
import os
from abjad.tools.systemtools.ImportManager import ImportManager
ImportManager.import_public_names_from_path_into_namespace(
    os.path.join(__path__[0], 'tools', 'exceptiontools'),
    __builtins__,
    )
del ImportManager
del os
del tools

# import version information
from abjad._version import __version_info__, __version__
del _version

def f(expr):
    print(format(expr, 'lilypond'))


# -*- encoding: utf-8 -*-
from abjad import *


def test_markuptools_MarkupInventory___format___01():

    inventory = markuptools.MarkupInventory(['foo', 'bar'])

    assert systemtools.TestManager.compare(
        format(inventory),
        r'''
        markuptools.MarkupInventory(
            [
                markuptools.Markup(
                    contents=('foo',),
                    ),
                markuptools.Markup(
                    contents=('bar',),
                    ),
                ]
            )
        ''',
        )


# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSegment_alpha_01():
    r'''Morris alpha transform of pitch-class segment.
    '''

    pitch_class_segment = pitchtools.PitchClassSegment([0, 6, 10, 4, 9, 2])
    assert pitch_class_segment.alpha() == pitchtools.PitchClassSegment(
        [1, 7, 11, 5, 8, 3])


# -*- encoding: utf-8 -*-
from abjad.tools import sequencetools


def register_pitch_class_numbers_by_pitch_number_aggregate(
    pitch_class_numbers, aggregate):
    '''Register `pitch_class_numbers` by pitch-number `aggregate`:

    ::

        >>> pitchtools.register_pitch_class_numbers_by_pitch_number_aggregate(
        ...     [10, 0, 2, 6, 8, 7, 5, 3, 1, 9, 4, 11],
        ...     [10, 19, 20, 23, 24, 26, 27, 29, 30, 33, 37, 40])
        [10, 24, 26, 30, 20, 19, 29, 27, 37, 33, 40, 23]

    Returns list of zero or more pitch numbers.
    '''

    if isinstance(pitch_class_numbers, list):
        result = [
            [p for p in aggregate if p % 12 == pc] for pc in [x % 12 for x in pitch_class_numbers]]
        result = sequencetools.flatten_sequence(result)
    elif isinstance(pitch_class_numbers, int):
        result = [p for p in aggregate if p % 12 == pitch_class_numbers][0]
    else:
        message = 'must be pitch-class number or list of pitch-class numbers.'
        raise TypeError(message)
    return result


# -*- encoding: utf-8 -*-


def apply_accidental_to_named_pitch(named_pitch, accidental=None):
    '''Apply `accidental` to `named_pitch`:

    ::

        >>> pitch = NamedPitch("cs''")
        >>> pitchtools.apply_accidental_to_named_pitch(pitch, 'f')
        NamedPitch("c''")

    Returns new named pitch.
    '''
    return named_pitch.apply_accidental(accidental)


# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class KeyCluster(AbjadObject):
    r'''A key cluster indication.

    ::

        >>> chord = Chord("<c' e' g' b' d'' f''>8")
        >>> key_cluster = indicatortools.KeyCluster()
        >>> attach(key_cluster, chord)
        >>> show(chord) # doctest: +SKIP

    ..  doctest::

        >>> print(format(chord))
        \once \override Accidental.stencil = ##f
        \once \override AccidentalCautionary.stencil = ##f
        \once \override Arpeggio.X-offset = #-2
        \once \override NoteHead.stencil = #ly:text-interface::print
        \once \override NoteHead.text = \markup {
            \filled-box #'(-0.6 . 0.6) #'(-0.7 . 0.7) #0.25
        }
        <c' e' g' b' d'' f''>8
            \markup {
                \center-align
                    \concat
                        {
                            \natural
                            \flat
                        }
                }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_include_black_keys',
        '_include_white_keys',
        '_markup_direction',
        '_suppress_markup',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        include_black_keys=True,
        include_white_keys=True,
        markup_direction=None,
        suppress_markup=False,
        ):
        assert include_black_keys or include_white_keys
        self._include_black_keys = bool(include_black_keys)
        self._include_white_keys = bool(include_white_keys)
        assert markup_direction in (None, Up, Center, Down)
        self._markup_direction = markup_direction
        self._suppress_markup = bool(suppress_markup)

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true when `expr` is a key cluster indication with black-key and
        white-key inclusion equal to that of this key cluster indication.
        Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if expr.include_black_keys == self.include_black_keys:
                if expr.include_white_keys == self.include_white_keys:
                    if expr.suppress_markup == self.suppress_markup:
                        return True
        return False

    def __hash__(self):
        r'''Hashes key cluster.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(KeyCluster, self).__hash__()

    ### PRIVATE PROPERTIES ###

    @property
    def _lilypond_format_bundle(self):
        from abjad.tools import markuptools
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.grob_overrides.append(
            '\\once \\override Accidental.stencil = ##f\n'
            '\\once \\override AccidentalCautionary.stencil = ##f\n'
            '\\once \\override Arpeggio.X-offset = #-2\n'
            '\\once \\override NoteHead.stencil = #ly:text-interface::print\n'
            '\\once \\override NoteHead.text = \markup {\n'
            "\t\\filled-box #'(-0.6 . 0.6) #'(-0.7 . 0.7) #0.25\n"
            '}'
            )
        if not self.suppress_markup:
            if self.include_black_keys and self.include_white_keys:
                string = r'\center-align \concat { \natural \flat }'
            elif self.include_black_keys:
                string = r'\center-align \flat'
            else:
                string = r'\center-align \natural'
            markup = markuptools.Markup(
                string,
                direction=self.markup_direction,
                )
            markup_format_pieces = markup._get_format_pieces()
            lilypond_format_bundle.right.markup.extend(markup_format_pieces)
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def include_black_keys(self):
        r'''Is true if key cluster includes black keys.

        Returns boolean.
        '''
        return self._include_black_keys

    @property
    def include_white_keys(self):
        r'''Is true if key cluster includes white keys.

        Returns boolean.
        '''
        return self._include_white_keys

    @property
    def markup_direction(self):
        r'''Gets markup direction.

        Returns ordinal constant or none.
        '''
        return self._markup_direction

    @property
    def suppress_markup(self):
        r'''Is true if key cluster suppresses key markup.

        Returns boolean.
        '''
        return self._suppress_markup



def set_(expr):
    r'''Applies LilyPond context setting to `expr`.

    Returns LilyPond context setting manager.
    '''
    from abjad.tools import lilypondnametools

    if getattr(expr, '_lilypond_setting_name_manager', None) is None:
        manager = lilypondnametools.LilyPondSettingNameManager()
        expr._lilypond_setting_name_manager = manager

    return expr._lilypond_setting_name_manager


# -*- encoding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools.rhythmtreetools import RhythmTreeNode


class QGridLeaf(RhythmTreeNode):
    r'''A leaf in a ``QGrid`` structure.

    ::

        >>> leaf = quantizationtools.QGridLeaf()

    ::

        >>> leaf
        QGridLeaf(
            preprolated_duration=Duration(1, 1),
            is_divisible=True
            )

    Used internally by ``QGrid``.
    '''

    ### INITIALIZER ###

    def __init__(
        self,
        preprolated_duration=1, q_event_proxies=None, is_divisible=True):
        from abjad.tools import quantizationtools
        RhythmTreeNode.__init__(self, preprolated_duration)
        if q_event_proxies is None:
            self._q_event_proxies = []
        else:
            assert all(isinstance(x, quantizationtools.QEventProxy)
                for x in q_event_proxies)
            self._q_event_proxies = list(q_event_proxies)
        self._is_divisible = bool(is_divisible)

    ### SPECIAL METHODS ###

    def __call__(self, pulse_duration):
        r'''Calls q-grid leaf.

        Returns selection of notes.
        '''
        pulse_duration = durationtools.Duration(pulse_duration)
        total_duration = pulse_duration * self.preprolated_duration
        return scoretools.make_notes(0, total_duration)

    def __eq__(self, expr):
        r'''Is true when `expr` is a q-grid leaf with preprolated duration,
        q-event proxies and divisibility flag equal to those of this q-grid
        leaf. Otherwise false.

        Returns boolean.
        '''
        if type(self) == type(expr):
            if self.preprolated_duration == expr.preprolated_duration:
                if self.q_event_proxies == expr.q_event_proxies:
                    if self._is_divisible == expr.is_divisible:
                        return True
        return False

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns tuple.
        '''
        return (
            self.preprolated_duration,
            tuple(self.q_event_proxies),
            self.is_divisible,
            )

    def __hash__(self):
        r'''Hashes q-grid leaf.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(QGridLeaf, self).__hash__()

    ### PRIVATE PROPERTIES ###

    @property
    def _pretty_rtm_format_pieces(self):
        return [str(self.preprolated_duration)]

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            keywords_ignored_when_false=(
                'q_event_proxies',
                ),
            )

    ### PUBLIC PROPERTIES ###

    @property
    def graphviz_graph(self):
        r'''Graphviz graph of q-grid leaf.

        Returns Graphviz graph.
        '''
        graph = documentationtools.GraphvizGraph(name='G')
        node = documentationtools.GraphvizNode(
            attributes={
                label: str(self.preprolated_duration),
                shape: 'box'
                }
            )
        graph.append(node)
        return graph

    @property
    def is_divisible(self):
        r'''Flag for whether the node may be further divided
        under some search tree.
        '''
        return self._is_divisible

    @is_divisible.setter
    def is_divisible(self, arg):
        self._is_divisible = bool(arg)

    @property
    def preceding_q_event_proxies(self):
        r'''Preceding q-event proxies of q-grid leaf.

        Returns list.
        '''
        return [x for x in self._q_event_proxies
            if x.offset < self.start_offset]

    @property
    def q_event_proxies(self):
        r'''Q-event proxies of q-grid leaf.
        '''
        return self._q_event_proxies

    @property
    def rtm_format(self):
        r'''RTM format of q-grid leaf.
        '''
        return str(self.preprolated_duration)

    @property
    def succeeding_q_event_proxies(self):
        r'''Succeeding q-event proxies of q-grid leaf.

        Returns list.
        '''
        return [x for x in self._q_event_proxies
            if self.start_offset <= x.offset]


# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_increase_elements_01():

    sequence_1 = range(10)
    sequence_2 = sequencetools.increase_elements(sequence_1, [2, 0])
    assert sequence_2 == [2, 1, 4, 3, 6, 5, 8, 7, 10, 9]


def test_sequencetools_increase_elements_02():

    sequence_1 = [1, 1, 2, 3, 5, 5, 1, 2, 5, 5, 6]
    sequence_2 = sequencetools.increase_elements(
        sequence_1, [0.5, 0.5], indices=[0, 4, 8]
        )
    assert sequence_2 == [1.5, 1.5, 2, 3, 5.5, 5.5, 1, 2, 5.5, 5.5, 6]


# -*- encoding: utf-8 -*-
from abjad import *


def test_sievetools_Sieve_from_cycle_tokens_01():
    r'''Single length-2 cycle token with modulo and residue list.
    '''

    cycle_token = (6, [0, 4, 5])
    sieve = sievetools.Sieve.from_cycle_tokens(cycle_token)
    RC = sievetools.ResidueClass

    assert sieve.rcs == [RC(6, 0), RC(6, 4), RC(6, 5)]


def test_sievetools_Sieve_from_cycle_tokens_02():
    r'''Single length-3 cycle token with modulo, residue list and offset.
    '''

    cycle_token = (6, [0, 4, 5], 1)
    sieve = sievetools.Sieve.from_cycle_tokens(cycle_token)
    RC = sievetools.ResidueClass

    assert sieve.rcs == [RC(6, 0), RC(6, 1), RC(6, 5)]


def test_sievetools_Sieve_from_cycle_tokens_03():
    r'''Large offset.
    '''

    cycle_token = (6, [0, 4, 5], 99)
    sieve = sievetools.Sieve.from_cycle_tokens(cycle_token)
    RC = sievetools.ResidueClass

    assert sieve.rcs == [RC(6, 1), RC(6, 2), RC(6, 3)]


def test_sievetools_Sieve_from_cycle_tokens_04():
    r'''Unsorted residues are sorted on initialization.
    '''

    cycle_token = (6, [5, 0, 4])
    sieve = sievetools.Sieve.from_cycle_tokens(cycle_token)
    RC = sievetools.ResidueClass

    assert sieve.rcs == [RC(6, 0), RC(6, 4), RC(6, 5)]

def test_sievetools_Sieve_from_cycle_tokens_05():
    r'''Arbitrarily many cycle tokens.
    '''

    cycle_token_1 = (6, [0, 4, 5])
    cycle_token_2 = (10, [0, 1, 2])
    sieve = sievetools.Sieve.from_cycle_tokens(cycle_token_1, cycle_token_2)

    "{RC(6, 0) | RC(6, 4) | RC(6, 5) | RC(10, 0) | RC(10, 1) | RC(10, 2)}"

    RC = sievetools.ResidueClass
    assert sieve.rcs == [
        RC(6, 0), RC(6, 4), RC(6, 5), RC(10, 0), RC(10, 1), RC(10, 2)]


def test_sievetools_Sieve_from_cycle_tokens_06():
    r'''Unsorted RCs are sorted on initialization.
    '''

    cycle_token_1 = (6, [0, 4, 5])
    cycle_token_2 = (10, [0, 1, 2])
    sieve = sievetools.Sieve.from_cycle_tokens(cycle_token_2, cycle_token_1)

    "{RC(6, 0) | RC(6, 4) | RC(6, 5) | RC(10, 0) | RC(10, 1) | RC(10, 2)}"

    RC = sievetools.ResidueClass
    assert sieve.rcs == [
        RC(6, 0), RC(6, 4), RC(6, 5), RC(10, 0), RC(10, 1), RC(10, 2)]


def test_sievetools_Sieve_from_cycle_tokens_07():
    r'''Cycle tokens allow optional offset values.
    '''

    cycle_token_1 = (6, [0, 4, 5])
    cycle_token_2 = (10, [0, 1, 2], 6)
    sieve = sievetools.Sieve.from_cycle_tokens(cycle_token_2, cycle_token_1)

    "{RC(6, 0) | RC(6, 4) | RC(6, 5) | RC(10, 6) | RC(10, 7) | RC(10, 8)}"

    RC = sievetools.ResidueClass
    assert sieve.rcs == [
        RC(6, 0), RC(6, 4), RC(6, 5), RC(10, 6), RC(10, 7), RC(10, 8)]


def test_sievetools_Sieve_from_cycle_tokens_08():
    r'''Works with no cycle tokens.
    '''

    sieve = sievetools.Sieve.from_cycle_tokens()
    assert sieve.rcs == []


def test_sievetools_Sieve_from_cycle_tokens_09():
    r'''Sieves count as cycle tokens in themselves.
    '''

    RC = sievetools.ResidueClass
    sieve_cycle_token = RC(6, 0) | RC(6, 1)
    sieve = sievetools.Sieve.from_cycle_tokens(sieve_cycle_token)

    assert sieve.rcs == [RC(6, 0), RC(6, 1)]


# -*- encoding: utf-8 -*-
from abjad.tools.durationtools.Duration import Duration


class Offset(Duration):
    '''A musical offset.

    ::

        >>> durationtools.Offset(121, 16)
        Offset(121, 16)

    Offset inherits from duration (which inherits from built-in ``Fraction``).
    '''

    ### SPECIAL METHODS ###

    def __sub__(self, expr):
        '''Offset taken from offset returns duration:

        ::

            >>> durationtools.Offset(2) - durationtools.Offset(1, 2)
            Duration(3, 2)

        Duration taken from offset returns another offset:

        ::

            >>> durationtools.Offset(2) - durationtools.Duration(1, 2)
            Offset(3, 2)

        Coerce `expr` to offset when `expr` is neither offset nor duration:

        ::

            >>> durationtools.Offset(2) - Fraction(1, 2)
            Duration(3, 2)

        Returns duration or offset.
        '''
        if isinstance(expr, type(self)):
            return Duration(Duration.__sub__(self, expr))
        elif isinstance(expr, Duration):
            return Duration.__sub__(self, expr)
        else:
            expr = type(self)(expr)
            return self - expr


# -*- encoding: utf-8 -*-


def select(expr=None, contiguous=False):
    r'''Selects `expr`.

    Returns selection.
    '''
    from abjad.tools import scoretools
    from abjad.tools import selectiontools
    from abjad.tools import spannertools
    Selection = selectiontools.Selection
    if contiguous:
        if isinstance(expr, (list, tuple)):
            assert Selection._all_are_contiguous_components_in_same_logical_voice(expr)
        return selectiontools.ContiguousSelection(expr)
    elif isinstance(expr, scoretools.Component):
        return selectiontools.ContiguousSelection(expr)
    elif hasattr(expr, '_music'):
        music = expr._music
        return selectiontools.Selection(music)
    elif isinstance(expr, spannertools.Spanner):
        music = expr._components
        return selectiontools.Selection(music)
    elif expr is None:
        return selectiontools.Selection()
    else:
        return selectiontools.Selection(expr)


# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools.spannertools.Spanner import Spanner


class Glissando(Spanner):
    r'''A glissando.

    ..  container:: example

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> glissando = spannertools.Glissando()
            >>> attach(glissando, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                c'8 \glissando
                d'8 \glissando
                e'8 \glissando
                f'8
            }

    Formats nonlast leaves in spanner with LilyPond ``\glissando`` command.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(
        self,
        overrides=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )

    ### PRIVATE METHODS ###

    def _get_lilypond_format_bundle(self, leaf):
        lilypond_format_bundle = self._get_basic_lilypond_format_bundle(leaf)
        prototype = (scoretools.Chord, scoretools.Note)
        if not self._is_my_last_leaf(leaf) and isinstance(leaf, prototype):
            string = r'\glissando'
            lilypond_format_bundle.right.spanner_starts.append(string)
        return lilypond_format_bundle



# -*- encoding: utf-8 -*-
from __future__ import print_function
import importlib
import inspect
import types
from abjad.tools.abctools.AbjadObject import AbjadObject


class InheritanceGraph(AbjadObject):
    r'''Generates a graph of a class or collection of
    classes as a dictionary of parent-children relationships:

    ::

        >>> class A(object): pass
        ...
        >>> class B(A): pass
        ...
        >>> class C(B): pass
        ...
        >>> class D(B): pass
        ...
        >>> class E(C, D): pass
        ...
        >>> class F(A): pass
        ...

    ::

        >>> graph = documentationtools.InheritanceGraph(addresses=(F, E))

    ``InheritanceGraph`` may be instantiated from one or more instances,
    classes or modules. If instantiated from a module, all public classes
    in that module will be taken into the graph.

    A `root_class` keyword may be defined at instantiation, which filters out
    all classes from the graph which do not inherit from that `root_class`
    (or are not already the `root_class`):

    ::

        >>> graph = documentationtools.InheritanceGraph(
        ...     (A, B, C, D, E, F), root_addresses=(B,))

    The class is intended for use in documenting packages.

    To document all of Abjad, use this formulation:

    ::

        >>> graph = documentationtools.InheritanceGraph(
        ...     addresses=('abjad',))

    To document only those classes descending from Container,
    use this formulation:

    ::

        >>> graph = documentationtools.InheritanceGraph(
        ...     addresses=('abjad',),
        ...     root_addresses=(Container,)
        ...     )

    To document only those classes whose lineage pass through scoretools,
    use this formulation:

    ::

        >>> graph = documentationtools.InheritanceGraph(
        ...     addresses=('abjad',),
        ...     lineage_addresses=(scoretools,),
        ...     )

    When creating the Graphviz representation, classes in the inheritance
    graph may be hidden, based on their distance from any defined lineage
    class:

    ::

        >>> graph = documentationtools.InheritanceGraph(
        ...     addresses=('abjad',),
        ...     lineage_addresses=(instrumenttools.Instrument,),
        ...     lineage_prune_distance=1,
        ...     )

    Returns ``InheritanceGraph`` instance.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_addresses',
        '_child_parents_mapping',
        '_immediate_classes',
        '_lineage_addresses',
        '_lineage_classes',
        '_lineage_distance_mapping',
        '_lineage_prune_distance',
        '_parent_children_mapping',
        '_recurse_into_submodules',
        '_root_addresses',
        '_root_classes',
        '_use_clusters',
        '_use_groups',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        addresses=('abjad',),
        lineage_addresses=None,
        lineage_prune_distance=None,
        recurse_into_submodules=True,
        root_addresses=None,
        use_clusters=True,
        use_groups=True,
        ):

        self._recurse_into_submodules = bool(recurse_into_submodules)
        if lineage_prune_distance is not None:
            lineage_prune_distance = int(lineage_prune_distance)
            assert 0 < lineage_prune_distance
        self._lineage_prune_distance = lineage_prune_distance
        self._use_clusters = bool(use_clusters)
        self._use_groups = bool(use_groups)

        # main addresses
        if addresses is None:
            addresses = ('abjad',)
        all_main_classes, main_immediate_classes, main_cached_addresses = \
            self._collect_classes(addresses, self.recurse_into_submodules)
        self._addresses = main_cached_addresses

        # lineage addresses
        if lineage_addresses is not None:
            all_lineage_classes, lineage_immediate_classes, lineage_cached_addresses = \
                self._collect_classes(lineage_addresses, False)
            self._lineage_addresses = lineage_cached_addresses
            self._lineage_classes = frozenset(all_lineage_classes)
        else:
            self._lineage_addresses = None
            self._lineage_classes = frozenset([])

        # root addresses
        if root_addresses is not None:
            all_root_classes, root_immediate_classes, root_cached_addresses = \
                self._collect_classes(root_addresses, False)
            self._root_addresses = root_cached_addresses
            self._root_classes = frozenset(all_root_classes)
        else:
            self._root_addresses = None
            self._root_classes = frozenset([object])

        child_parents_mapping, parent_children_mapping = \
            self._build_basic_mappings(all_main_classes)
        self._strip_nonlineage_classes(
            child_parents_mapping, parent_children_mapping)

        self._child_parents_mapping = child_parents_mapping
        self._parent_children_mapping = parent_children_mapping

        self._immediate_classes = main_immediate_classes.intersection(
            self._parent_children_mapping.keys())

        self._lineage_distance_mapping = self._find_lineage_distances()

    ### PRIVATE METHODS ###

    def _build_basic_mappings(self, classes):
        child_parents_mapping = {}
        parent_children_mapping = {}
        invalid_classes = set([])
        def recurse(current_class):
            if current_class in child_parents_mapping:
                return True
            elif current_class in invalid_classes:
                return False
            mro = list(inspect.getmro(current_class))
            while len(mro) and mro[-1] not in self.root_classes:
                mro.pop()
            if not mro:
                invalid_classes.add(current_class)
                return False
            parents = [x for x in current_class.__bases__ if recurse(x)]
            child_parents_mapping[current_class] = set(parents)
            parent_children_mapping[current_class] = set([])
            for parent in parents:
                parent_children_mapping[parent].add(current_class)
            return True
        for current_class in classes:
            recurse(current_class)
        return child_parents_mapping, parent_children_mapping

    def _collect_classes(self, addresses, recurse_into_submodules):
        all_classes = set([])
        cached_addresses = []
        immediate_classes = set([])
        visited_modules = set([])
        assert 0 < len(addresses)
        for x in addresses:
            if isinstance(x, (str, types.ModuleType)):
                if isinstance(x, types.ModuleType):
                    module = x
                else:
                    module = importlib.import_module(x)
                for y in module.__dict__.values():
                    if isinstance(y, type):
                        all_classes.add(y)
                        immediate_classes.add(y)
                    elif isinstance(y, types.ModuleType) and \
                        recurse_into_submodules:
                        all_classes.update(
                            self._submodule_recurse(y, visited_modules))
                address = module.__name__
            else:
                if isinstance(x, type):
                    current_class = x
                elif isinstance(x, tuple) and len(x) == 2:
                    module_name, class_name = x
                    module = importlib.import_module(module_name)
                    current_class = getattr(module, class_name)
                else:
                    current_class = x.__class__
                all_classes.add(current_class)
                immediate_classes.add(current_class)
                address = (current_class.__module__, current_class.__name__)
            cached_addresses.append(address)
        return all_classes, immediate_classes, tuple(cached_addresses)

    def _find_lineage_distances(self):
        if not self.lineage_classes:
            return None
        if not self.lineage_prune_distance:
            return None
        distance_mapping = {}
        def recurse_downward(current_class, distance=0):
            if current_class not in self.parent_children_mapping:
                return
            for child in self.parent_children_mapping[current_class]:
                if child not in distance_mapping:
                    distance_mapping[child] = distance + 1
                    recurse_downward(child, distance + 1)
                elif (distance + 1) < distance_mapping[child]:
                    distance_mapping[child] = distance + 1
                    recurse_downward(child, distance + 1)
        for current_class in self.lineage_classes:
            recurse_downward(current_class)
        return distance_mapping

    def _strip_nonlineage_classes(self,
        child_parents_mapping, parent_children_mapping):
        if not self.lineage_classes:
            return
        def recurse_upward(current_class, invalid_classes):
            if current_class not in child_parents_mapping:
                return
            for parent in child_parents_mapping[current_class]:
                if parent in invalid_classes:
                    invalid_classes.remove(parent)
                    recurse_upward(parent, invalid_classes)
        # TODO: change nested helper to private static method
        def recurse_downward(current_class, invalid_classes):
            if current_class not in parent_children_mapping:
                return
            for child in parent_children_mapping[current_class]:
                if child in invalid_classes:
                    invalid_classes.remove(child)
                    recurse_downward(child, invalid_classes)
        invalid_classes = set(
            list(child_parents_mapping.keys()) + list(parent_children_mapping.keys()))
        for current_class in self.lineage_classes:
            if current_class in invalid_classes:
                invalid_classes.remove(current_class)
            recurse_upward(current_class, invalid_classes)
            recurse_downward(current_class, invalid_classes)
        for current_class in invalid_classes:
            for child in parent_children_mapping[current_class]:
                child_parents_mapping[child].remove(current_class)
            for parent in child_parents_mapping[current_class]:
                parent_children_mapping[parent].remove(current_class)
            del(parent_children_mapping[current_class])
            del(child_parents_mapping[current_class])

    def _submodule_recurse(self, module, visited_modules):
        result = []
        for obj in list(module.__dict__.values()):
            if isinstance(obj, type):
                result.append(obj)
            elif isinstance(obj, types.ModuleType) and \
                obj not in visited_modules:
                visited_modules.add(obj)
                result.extend(self._submodule_recurse(obj, visited_modules))
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def addresses(self):
        r'''Addresses of inheritance graph.
        '''
        return self._addresses

    @property
    def child_parents_mapping(self):
        r'''Child / parent mapping of inheritance graph.
        '''
        return self._child_parents_mapping

    @property
    def graphviz_format(self):
        r'''Graphviz format of inheritance graph.
        '''
        return self.graphviz_graph.graphviz_format

    @property
    def graphviz_graph(self):
        r'''Graphviz graph of inheritance graph.
        '''
        from abjad.tools import documentationtools

        # TODO: change nested helper to private static method
        def get_class_name_pieces(current_class):
            parts = (current_class.__module__ + '.' + current_class.__name__).split('.')
            name = [parts[0]]
            for part in parts[1:]:
                if part != name[-1]:
                    name.append(part)
            if name[0] in ('abjad', 'experimental'):
                return name[2:]
            return name

        class_nodes = {}

        graph = documentationtools.GraphvizGraph(
            name='InheritanceGraph',
            attributes={
                'color': 'lightslategrey',
                'fontname': 'Arial',
                'outputorder': 'edgesfirst',
                'overlap': 'prism',
                'penwidth': 2,
                #'ranksep': 0.5,
                'splines': 'spline',
                'style': ('dotted', 'rounded'),
            },
            edge_attributes={
                'color': 'lightsteelblue2',
                'penwidth': 2,
            },
            node_attributes={
                'colorscheme': 'pastel19',
                'fontname': 'Arial',
                'fontsize': 12,
                'penwidth': 2,
                'style': ('filled', 'rounded'),
            },
            )

        for current_class in sorted(self.parent_children_mapping,
            key=lambda x: (x.__module__, x.__name__)):
            pieces = get_class_name_pieces(current_class)

            try:
                cluster = graph[pieces[0]]
            except KeyError:
                cluster = documentationtools.GraphvizSubgraph(
                    name=pieces[0],
                    attributes={
                        'label': pieces[0],
                    },
                    )
                graph.append(cluster)

            node = documentationtools.GraphvizNode(
                name='.'.join(pieces),
                )
            #node.attributes['label'] = '\\n'.join(pieces)
            node.attributes['label'] = pieces[-1]

            if current_class in self.immediate_classes:
                pass
            if current_class in self.root_classes:
                pass
            if inspect.isabstract(current_class):
                node.attributes['shape'] = 'oval'
                node.attributes['style'] = 'bold'
            else:
                node.attributes['shape'] = 'box'
            if current_class in self.lineage_classes:
                node.attributes['color'] = 'black'
                node.attributes['fontcolor'] = 'white'
                node.attributes['style'] = ('filled', 'rounded')

            if self.lineage_prune_distance is None:
                cluster.append(node)
                class_nodes[current_class] = node
            elif current_class not in self.lineage_distance_mapping:
                cluster.append(node)
                class_nodes[current_class] = node
            else:
                ok_distance = self.lineage_prune_distance + 1
                distance = self.lineage_distance_mapping[current_class]
                if distance < ok_distance:
                    cluster.append(node)
                    class_nodes[current_class] = node
                elif distance == ok_distance:
                    node.attributes['shape'] = 'invis'
                    node.attributes['style'] = 'transparent'
                    node.attributes['label'] = ' '
                    cluster.append(node)
                    class_nodes[current_class] = node
                elif ok_distance < distance:
                    pass

        distances = self.lineage_distance_mapping
        for parent, children in self.parent_children_mapping.items():
            for child in children:
                ok_to_join = False
                if self.lineage_prune_distance is None:
                    ok_to_join = True
                elif parent not in distances:
                    if child not in distances:
                        ok_to_join = True
                    elif child in distances and \
                        distances[child] <= ok_distance:
                        ok_to_join = True
                elif child not in distances:
                    if parent not in distances:
                        ok_to_join = True
                    elif parent in distances and \
                        distances[parent] <= ok_distance:
                        ok_to_join = True
                elif distances[child] <= ok_distance and \
                    distances[parent] <= ok_distance:
                    ok_to_join = True
                if ok_to_join:
                    parent_node = class_nodes[parent]
                    child_node = class_nodes[child]
                    documentationtools.GraphvizEdge()(parent_node, child_node)

        for i, cluster in enumerate(
            sorted(graph.children, key=lambda x: x.name)):
            color = i % 9 + 1
            for node in cluster:
                if 'color' not in node.attributes:
                    node.attributes['color'] = color
                if self.use_groups:
                    node.attributes['group'] = i
            if not self.use_clusters:
                graph.extend(cluster[:])
                graph.remove(cluster)

        if self.root_addresses is None:
            graph.attributes['root'] = '__builtin__.object'

        return graph

    @property
    def immediate_classes(self):
        r'''Immediate classes of inheritance graph.
        '''
        return self._immediate_classes

    @property
    def lineage_addresses(self):
        r'''Lineage addresses of inheritance graph.
        '''
        return self._lineage_addresses

    @property
    def lineage_classes(self):
        r'''Lineage classes of inheritance graph.
        '''
        return self._lineage_classes

    @property
    def lineage_distance_mapping(self):
        r'''Lineage distance mapping of inheritance graph.
        '''
        return self._lineage_distance_mapping

    @property
    def lineage_prune_distance(self):
        r'''Lineage prune distance of inheritance graph.
        '''
        return self._lineage_prune_distance

    @property
    def parent_children_mapping(self):
        r'''Parent / children mapping of inheritancee graph.
        '''
        return self._parent_children_mapping

    @property
    def recurse_into_submodules(self):
        r'''Recurse into submodules.
        '''
        return self._recurse_into_submodules

    @property
    def root_addresses(self):
        r'''Root addresses of inheritance graph.
        '''
        return self._root_addresses

    @property
    def root_classes(self):
        r'''Root classes of inheritance graph.
        '''
        return self._root_classes

    @property
    def use_clusters(self):
        r'''Use clusters.
        '''
        return self._use_clusters

    @property
    def use_groups(self):
        r'''Use groups.
        '''
        return self._use_groups



# -*- encoding: utf-8 -*-
from abjad.tools.topleveltools import override


def make_rhythmic_sketch_staff(music):
    r'''Make rhythmic staff with transparent time_signature and transparent bar
    lines.
    '''
    from abjad.tools import scoretools

    staff = scoretools.Staff(music)
    staff.context_name = 'RhythmicStaff'
    override(staff).time_signature.transparent = True
    override(staff).bar_line.transparent = True

    return staff


# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_Sequence_reverse_01():
    r'''Reverse sequence.
    '''

    assert sequencetools.Sequence(1, 2, 3, 4, 5).reverse() == \
        sequencetools.Sequence(5, 4, 3, 2, 1)
    assert sequencetools.Sequence(1, 2, 3, 4, 5).reverse() == \
        sequencetools.Sequence(5, 4, 3, 2, 1)


# -*- encoding: utf-8 -*-
from abjad import *


def test_labeltools_label_leaves_in_expr_with_named_intervals_01():

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
    labeltools.label_leaves_in_expr_with_named_intervals(staff)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 ^ \markup { +M2 }
            d'8 ^ \markup { +M2 }
            e'8 ^ \markup { +m2 }
            f'8 ^ \markup { +M2 }
            g'8 ^ \markup { +M2 }
            a'8 ^ \markup { +M2 }
            b'8 ^ \markup { +m2 }
            c''8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_labeltools_label_leaves_in_expr_with_named_intervals_02():

    pitch_numbers = [0, 13, 11, 8, 2, 3, 9, 10, 6, 5]
    staff = Staff(scoretools.make_notes(pitch_numbers, [Duration(1, 8)]))
    labeltools.label_leaves_in_expr_with_named_intervals(staff)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 ^ \markup { +aug8 }
            cs''8 ^ \markup { -M2 }
            b'8 ^ \markup { -aug2 }
            af'8 ^ \markup { -dim5 }
            d'8 ^ \markup { +m2 }
            ef'8 ^ \markup { +aug4 }
            a'8 ^ \markup { +m2 }
            bf'8 ^ \markup { -dim4 }
            fs'8 ^ \markup { -aug1 }
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


# -*- encoding: utf-8 -*-
from abjad import *


def test_markuptools_make_blank_line_markup_01():

    markup = markuptools.make_blank_line_markup()

    assert systemtools.TestManager.compare(
        format(markup, 'lilypond'),
        r'''
        \markup {
            \fill-line
                {
                    " "
                }
            }
        ''',
        )


# -*- encoding: utf-8 -*-
from abjad import *


def test_spannertools_Slur_direction_01():

    container = Container("c'8 d'8 e'8 f'8")
    slur = Slur(direction=Up)
    attach(slur, container)

    assert systemtools.TestManager.compare(
        container,
        r'''
        {
            c'8 ^ (
            d'8
            e'8
            f'8 )
        }
        '''
        )

    assert inspect_(container).get_spanners() == set([slur])


# -*- encoding: utf-8 -*-


def are_relatively_prime(expr):
    '''Is true when `expr` is a sequence comprising zero or more numbers,
    all of which are relatively prime.

    ::

        >>> mathtools.are_relatively_prime([13, 14, 15])
        True

    Otherwise false:

    ::

        >>> mathtools.are_relatively_prime([13, 14, 15, 16])
        False

    Returns true when `expr` is an empty sequence:

    ::

        >>> mathtools.are_relatively_prime([])
        True

    Returns false when `expr` is nonsensical type:

    ::

        >>> mathtools.are_relatively_prime('foo')
        False

    Returns boolean.
    '''
    from abjad.tools import mathtools

    try:
        all_factors = set([])
        for number in expr:
            current_factors = mathtools.factors(number)
            current_factors.remove(1)
            current_factors = set(current_factors)
            if all_factors & current_factors:
                return False
            all_factors.update(current_factors)
        return True
    # TODO: remove unqualified except
    except:
        return False


# -*- encoding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools.abctools import AbjadValueObject
from abjad.tools.topleveltools import new


class BurnishSpecifier(AbjadValueObject):
    r'''Burnish specifier.

    ..  container:: example

        Force first leaf of each division to be a rest:

        ::

            >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
            ...     lefts=(-1,),
            ...     left_lengths=(1,),
            ...     )

    ..  container:: example

        Force the first three leaves of each division to be rests:

        ::

            >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
            ...     lefts=(-1,),
            ...     left_lengths=(3,),
            ...     )

    ..  container:: example

        Force last leaf of each division to be a rest:

        ::

            >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
            ...     rights=(-1,),
            ...     right_lengths=(1,),
            ...     )

    ..  container:: example

        Force the last three leaves of each division to be rests:

        ::

            >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
            ...     rights=(-1,),
            ...     right_lengths=(3,),
            ...     )

    ..  container:: example

        Force the first leaf of every even-numbered division to be a rest;
        force the first leaf of every odd-numbered division to be a note.

        ::

            >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
            ...     lefts=(-1, 1),
            ...     left_lengths=(1,),
            ...     )

    ..  container:: example

        Force the last leaf of every even-numbered division to be a rest;
        force the last leaf of every odd-numbered division to be a note.

        ::

            >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
            ...     rights=(-1, 1),
            ...     right_lengths=(1,),
            ...     )

    ..  container:: example

        Force the first leaf of every even-numbered division to be a rest;
        leave the first leaf of every odd-numbered division unchanged.

        ::

            >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
            ...     lefts=(-1, 0),
            ...     left_lengths=(1,),
            ...     )

    ..  container:: example

        Force the last leaf of every even-numbered division to be a rest;
        leave the last leaf of every odd-numbered division unchanged.

        ::

            >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
            ...     rights=(-1, 0),
            ...     right_lengths=(1,),
            ...     )

    Burnish specifiers are immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_burnish_divisions',
        '_burnish_output',
        '_left_lengths',
        '_lefts',
        '_middles',
        '_right_lengths',
        '_rights',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        burnish_divisions=False,
        burnish_output=False,
        lefts=None,
        middles=None,
        rights=None,
        left_lengths=None,
        right_lengths=None,
        ):
        assert isinstance(burnish_divisions, bool)
        assert isinstance(burnish_output, bool)
        self._burnish_divisions = burnish_divisions
        self._burnish_output = burnish_output
        lefts = self._to_tuple(lefts)
        middles = self._to_tuple(middles)
        rights = self._to_tuple(rights)
        left_lengths = self._to_tuple(left_lengths)
        right_lengths = self._to_tuple(right_lengths)
        assert self._is_sign_tuple(lefts)
        assert self._is_sign_tuple(middles)
        assert self._is_sign_tuple(rights)
        assert self._is_length_tuple(left_lengths)
        assert self._is_length_tuple(right_lengths)
        self._lefts = lefts
        self._middles = middles
        self._rights = rights
        self._left_lengths = left_lengths
        self._right_lengths = right_lengths

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true when `expr` is a burnish specifier with input parameters
        equal to those of this burnish specifier. Otherwise false.

        ..  container:: example

            ::

                >>> burnish_specifier_1 = rhythmmakertools.BurnishSpecifier(
                ...     lefts=(-1, 0),
                ...     left_lengths=(1,)
                ... )
                >>> burnish_specifier_2 = rhythmmakertools.BurnishSpecifier(
                ...     lefts=(-1, 0),
                ...     left_lengths=(1,)
                ... )
                >>> burnish_specifier_3 = rhythmmakertools.BurnishSpecifier()

            ::

                >>> burnish_specifier_1 == burnish_specifier_1
                True
                >>> burnish_specifier_1 == burnish_specifier_2
                True
                >>> burnish_specifier_1 == burnish_specifier_3
                False
                >>> burnish_specifier_2 == burnish_specifier_1
                True
                >>> burnish_specifier_2 == burnish_specifier_2
                True
                >>> burnish_specifier_2 == burnish_specifier_3
                False
                >>> burnish_specifier_3 == burnish_specifier_1
                False
                >>> burnish_specifier_3 == burnish_specifier_2
                False
                >>> burnish_specifier_3 == burnish_specifier_3
                True

        Returns boolean.
        '''
        from abjad.tools import systemtools
        return systemtools.StorageFormatManager.compare(self, expr)

    def __format__(self, format_specification=''):
        r'''Formats burnish specifier.

        ..  container:: example

            ::

                >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
                ...     lefts=(-1, 0),
                ...     left_lengths=(1,),
                ...     )

            ::

                >>> print(format(burnish_specifier))
                rhythmmakertools.BurnishSpecifier(
                    burnish_divisions=False,
                    burnish_output=False,
                    lefts=(-1, 0),
                    left_lengths=(1,),
                    )

        Returns string.
        '''
        return AbjadValueObject.__format__(
            self,
            format_specification=format_specification,
            )

    def __hash__(self):
        r'''Hashes burnish specifier.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(BurnishSpecifier, self).__hash__()

    def __ne__(self, expr):
        r'''Is true when `expr` does not equal burnish specifier.

        ..  container:: example

            ::

                >>> burnish_specifier_1 = rhythmmakertools.BurnishSpecifier(
                ...     lefts=(-1, 0),
                ...     left_lengths=(1,)
                ... )
                >>> burnish_specifier_2 = rhythmmakertools.BurnishSpecifier(
                ...     lefts=(-1, 0),
                ...     left_lengths=(1,)
                ... )
                >>> burnish_specifier_3 = rhythmmakertools.BurnishSpecifier()

            ::

                >>> burnish_specifier_1 != burnish_specifier_1
                False
                >>> burnish_specifier_1 != burnish_specifier_2
                False
                >>> burnish_specifier_1 != burnish_specifier_3
                True
                >>> burnish_specifier_2 != burnish_specifier_1
                False
                >>> burnish_specifier_2 != burnish_specifier_2
                False
                >>> burnish_specifier_2 != burnish_specifier_3
                True
                >>> burnish_specifier_3 != burnish_specifier_1
                True
                >>> burnish_specifier_3 != burnish_specifier_2
                True
                >>> burnish_specifier_3 != burnish_specifier_3
                False

        Returns boolean.
        '''
        return AbjadValueObject.__ne__(self, expr)

    def __repr__(self):
        r'''Gets interpreter representation of burnish specifier.

        ..  container:: example

            ::

                >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
                ...     lefts=(-1, 0),
                ...     left_lengths=(1,),
                ...     )

            ::

                >>> burnish_specifier
                BurnishSpecifier(burnish_divisions=False, burnish_output=False, lefts=(-1, 0), left_lengths=(1,))

        Returns string.
        '''
        return AbjadValueObject.__repr__(self)

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import systemtools
        from scoremanager import idetools
        return systemtools.AttributeManifest(
            systemtools.AttributeDetail(
                name='burnish_divisions',
                command='bd',
                editor=idetools.getters.get_boolean,
                ),
            systemtools.AttributeDetail(
                name='burnish_output',
                command='bo',
                editor=idetools.getters.get_boolean,
                ),
            systemtools.AttributeDetail(
                name='lefts',
                command='l',
                editor=idetools.getters.get_integers,
                ),
            systemtools.AttributeDetail(
                name='middles',
                command='m',
                editor=idetools.getters.get_integers,
                ),
            systemtools.AttributeDetail(
                name='rights',
                command='r',
                editor=idetools.getters.get_integers,
                ),
            systemtools.AttributeDetail(
                name='left_lengths',
                command='ll',
                editor=idetools.getters.get_integers,
                ),
            systemtools.AttributeDetail(
                name='right_lengths',
                command='rl',
                editor=idetools.getters.get_integers,
                ),
            )

    ### PRIVATE METHODS ###

    @staticmethod
    def _is_length_tuple(expr):
        if expr is None:
            return True
        if mathtools.all_are_nonnegative_integer_equivalent_numbers(expr):
            if isinstance(expr, tuple):
                return True
        return False

    @staticmethod
    def _is_sign_tuple(expr):
        if expr is None:
            return True
        if isinstance(expr, tuple):
            return all(x in (-1, 0, 1) for x in expr)
        return False

    @staticmethod
    def _to_tuple(expr):
        if isinstance(expr, list):
            expr = tuple(expr)
        return expr

    ### PUBLIC PROPERTIES ###

    @property
    def burnish_divisions(self):
        r'''Is true when rhythm-maker should burnish every division in output.
        Otherwise false.

        Defaults to false.

        Returns boolean.
        '''
        return self._burnish_divisions

    @property
    def burnish_output(self):
        r'''Is true when rhythm-maker should burnish first and last division
        in output. Otherwise false.

        Defaults to false.

        Returns boolean.
        '''
        return self._burnish_output

    @property
    def left_lengths(self):
        r'''Gets left lengths of burnish specifier.

        ..  container:: example

            ::

                >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
                ...     lefts=(-1, 0),
                ...     middles=(0,),
                ...     rights=(-1, -1, 0),
                ...     left_lengths=(2,),
                ...     right_lengths=(1,),
                ...     )

            ::

                >>> burnish_specifier.left_lengths
                (2,)

        Returns tuple or none.
        '''
        return self._left_lengths

    @property
    def lefts(self):
        r'''Gets lefts of burnish specifier.

        ..  container:: example

            ::

                >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
                ...     lefts=(-1, 0),
                ...     middles=(0,),
                ...     rights=(-1, -1, 0),
                ...     left_lengths=(2,),
                ...     right_lengths=(1,),
                ...     )

            ::

                >>> burnish_specifier.lefts
                (-1, 0)

        Returns tuple or none.
        '''
        return self._lefts

    @property
    def middles(self):
        r'''Gets middles of burnish specifier.

        ..  container:: example

            ::

                >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
                ...     lefts=(-1, 0),
                ...     middles=(0,),
                ...     rights=(-1, -1, 0),
                ...     left_lengths=(2,),
                ...     right_lengths=(1,),
                ...     )

            ::

                >>> burnish_specifier.middles
                (0,)

        Returns tuple or none.
        '''
        return self._middles

    @property
    def right_lengths(self):
        r'''Gets right lengths of burnish specifier.

        ..  container:: example

            ::

                >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
                ...     lefts=(-1, 0),
                ...     middles=(0,),
                ...     rights=(-1, -1, 0),
                ...     left_lengths=(2,),
                ...     right_lengths=(1,),
                ...     )

            ::

                >>> burnish_specifier.right_lengths
                (1,)

        Returns tuple or none.
        '''
        return self._right_lengths

    @property
    def rights(self):
        r'''Gets rights of burnish specifier.

        ..  container:: example

            ::

                >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
                ...     lefts=(-1, 0),
                ...     middles=(0,),
                ...     rights=(-1, -1, 0),
                ...     left_lengths=(2,),
                ...     right_lengths=(1,),
                ...     )

            ::

                >>> burnish_specifier.rights
                (-1, -1, 0)

        Returns tuple or none.
        '''
        return self._rights

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses burnish specification.

        ..  container:: example

            ::

                >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
                ...     burnish_divisions=True,
                ...     burnish_output=False,
                ...     lefts=(-1, 0),
                ...     middles=(0,),
                ...     rights=(-1, -1, 0),
                ...     left_lengths=(2,),
                ...     right_lengths=(1,),
                ...     )

            ::

                >>> print(format(burnish_specifier.reverse()))
                rhythmmakertools.BurnishSpecifier(
                    burnish_divisions=True,
                    burnish_output=False,
                    lefts=(0, -1),
                    middles=(0,),
                    rights=(0, -1, -1),
                    left_lengths=(2,),
                    right_lengths=(1,),
                    )

        Returns new burnish specification.
        '''
        from abjad.tools import rhythmmakertools
        lefts = rhythmmakertools.RhythmMaker._reverse_tuple(
            self.lefts)
        middles = rhythmmakertools.RhythmMaker._reverse_tuple(
            self.middles)
        rights = rhythmmakertools.RhythmMaker._reverse_tuple(
            self.rights)
        left_lengths = rhythmmakertools.RhythmMaker._reverse_tuple(
            self.left_lengths)
        right_lengths = rhythmmakertools.RhythmMaker._reverse_tuple(
            self.right_lengths)
        result = new(
            self,
            lefts=lefts,
            middles=middles,
            rights=rights,
            left_lengths=left_lengths,
            right_lengths=right_lengths,
            )
        return result

    def rotate(self, n=0):
        r'''Rotates burnish specification.

        ..  container:: example

            ::

                >>> burnish_specifier = rhythmmakertools.BurnishSpecifier(
                ...     burnish_divisions=True,
                ...     burnish_output=False,
                ...     lefts=(-1, 0),
                ...     middles=(0,),
                ...     rights=(-1, -1, 0),
                ...     left_lengths=(2,),
                ...     right_lengths=(1, 2, 3),
                ...     )

            ::

                >>> print(format(burnish_specifier.rotate(1)))
                rhythmmakertools.BurnishSpecifier(
                    burnish_divisions=True,
                    burnish_output=False,
                    lefts=(0, -1),
                    middles=(0,),
                    rights=(0, -1, -1),
                    left_lengths=(2,),
                    right_lengths=(3, 1, 2),
                    )

        Returns new burnish specification.
        '''
        from abjad.tools import rhythmmakertools
        lefts = rhythmmakertools.RhythmMaker._rotate_tuple(
            self.lefts, n)
        middles = rhythmmakertools.RhythmMaker._rotate_tuple(
            self.middles, n)
        rights = rhythmmakertools.RhythmMaker._rotate_tuple(
            self.rights, n)
        left_lengths = rhythmmakertools.RhythmMaker._rotate_tuple(
            self.left_lengths, n)
        right_lengths = rhythmmakertools.RhythmMaker._rotate_tuple(
            self.right_lengths, n)
        return new(
            self,
            lefts=lefts,
            middles=middles,
            rights=rights,
            left_lengths=left_lengths,
            right_lengths=right_lengths,
            )


# -*- encoding: utf-8 -*-
import sys
from abjad import *
from abjad.tools.mathtools import NonreducedFraction


def test_mathtools_NonreducedFraction___add___01():

    result = NonreducedFraction(1, 4) + NonreducedFraction(2, 8)
    assert result.pair == (4, 8)

    result = NonreducedFraction(2, 8) + NonreducedFraction(1, 4)
    assert result.pair == (4, 8)


def test_mathtools_NonreducedFraction___add___02():

    result = NonreducedFraction(3, 3) + 1
    assert result.pair == (6, 3)

    result = 1 + NonreducedFraction(3, 3)
    assert result.pair == (6, 3)


def test_mathtools_NonreducedFraction___add___03():

    a = mathtools.NonreducedFraction(3, 6)
    b = mathtools.NonreducedFraction(3, 12)

    result_one = systemtools.IOManager.count_function_calls(
        'a + b', locals())
    result_two = systemtools.IOManager.count_function_calls(
        'a + 10', locals())

    if sys.version_info[0] == 2:
        assert result_one == 67
        assert result_two == 35
    else:
        assert result_one == 82
        assert result_two == 40



# -*- encoding: utf-8 -*-
import functools
import inspect
import pytest
import abjad
from abjad.tools import datastructuretools
from abjad.tools import documentationtools
from abjad.tools import durationtools
from abjad.tools import mathtools


classes = documentationtools.list_all_abjad_classes()
@pytest.mark.parametrize('class_', classes)
def test_abjad___init___01(class_):
    r'''All concrete classes initialize from empty input.
    '''

    if not inspect.isabstract(class_):
        instance = class_()


valid_types = (
    bool,
    float,
    int,
    str,
    tuple,
    type(None),
    datastructuretools.OrdinalConstant,
    durationtools.Duration,
    mathtools.Infinity,
    mathtools.NegativeInfinity,
    )

classes = documentationtools.list_all_abjad_classes()
@pytest.mark.parametrize('obj', classes)
def test_abjad___init___02(obj):
    r'''Make sure class initializer keyword argument values are immutable.
    '''

    for attr in inspect.classify_class_attrs(obj):
        if attr.defining_class is not obj:
            continue
        elif attr.kind != 'method':
            continue
        obj = attr.object
        if isinstance(obj, functools.partial):
            obj = obj.function
        argument_specification = inspect.getargspec(obj)
        keyword_argument_names = argument_specification.args[1:]
        keyword_argument_values = argument_specification.defaults
        if keyword_argument_values is None:
            continue
        for name, value in zip(
            keyword_argument_names, keyword_argument_values):
            assert isinstance(value, valid_types), (attr.name, name, value)
            if isinstance(value, tuple):
                assert all(isinstance(x, valid_types) for x in value)


functions = documentationtools.list_all_abjad_functions()
@pytest.mark.parametrize('obj', functions)
def test_abjad___init___03(obj):
    r'''Make sure function keyword argument values are immutable.
    '''
    if isinstance(obj, functools.partial):
        obj = obj.function
    argument_specification = inspect.getargspec(obj)
    keyword_argument_names = argument_specification.args[1:]
    keyword_argument_values = argument_specification.defaults
    if keyword_argument_values is None:
        return
    for name, value in zip(
        keyword_argument_names, keyword_argument_values):
        assert isinstance(value, valid_types), (name, value)
        if isinstance(value, tuple):
            assert all(isinstance(x, valid_types) for x in value)


# -*- encoding: utf-8 -*-
from abjad import *


def test_indicatortools_ClefInventory___contains___01():

    inventory = indicatortools.ClefInventory(['treble', 'bass'])

    assert 'treble' in inventory
    assert Clef('treble') in inventory

    assert 'alto' not in inventory
    assert Clef('alto') not in inventory


# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools.topleveltools import iterate


def color_leaves_in_expr(expr, color):
    r"""Color leaves in `expr`:

    ::

        >>> staff = Staff("cs'8. [ r8. s8. <c' cs' a'>8. ]")

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            cs'8. [
            r8.
            s8.
            <c' cs' a'>8. ]
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> labeltools.color_leaves_in_expr(staff, 'red')

    ..  doctest::

        >>> print(format(staff))
        \new Staff {
            \once \override Accidental #'color = #red
            \once \override Beam #'color = #red
            \once \override Dots #'color = #red
            \once \override NoteHead #'color = #red
            \once \override Stem #'color = #red
            cs'8. [
            \once \override Dots #'color = #red
            \once \override Rest #'color = #red
            r8.
            s8.
            \once \override Accidental #'color = #red
            \once \override Beam #'color = #red
            \once \override Dots #'color = #red
            \once \override NoteHead #'color = #red
            \once \override Stem #'color = #red
            <c' cs' a'>8. ]
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Returns none.
    """
    from abjad.tools import labeltools

    for leaf in iterate(expr).by_class(scoretools.Leaf):
        labeltools.color_leaf(leaf, color)



# -*- encoding: utf-8 -*-
import copy
import inspect
import pytest
import abjad
from abjad.tools import documentationtools
from abjad.tools import scoretools


classes = documentationtools.list_all_abjad_classes()
@pytest.mark.parametrize('class_', classes)
def test_abjad___copy___01(class_):
    r'''All concrete classes with a storage format can copy.
    '''

    if not '_storage_format_specification' in dir(class_):
        return
    if inspect.isabstract(class_):
        return
    instance_one = class_()
    instance_two = copy.copy(instance_one)
    instance_one_format = format(instance_one, 'storage')
    instance_two_format = format(instance_two, 'storage')
    if not issubclass(class_, scoretools.Container):
        assert instance_one_format == instance_two_format
    # TODO: eventually this second asset should also pass
    #assert instance_one == instance_two



# -*- encoding: utf-8 -*-
from abjad import *


def test_datastructuretools_PayloadTree___init___01():
    r'''Initializes tree from other tree.
    '''

    tree_1 = datastructuretools.PayloadTree([[4, 5], [6, 7]])
    tree_2 = datastructuretools.PayloadTree(tree_1)

    assert tree_1 is not tree_2
    assert tree_1 == tree_2


def test_datastructuretools_PayloadTree___init___02():
    r'''Initializes correctly with strings.
    '''

    items = [['some', 'text'], ['more', 'text']]
    tree = datastructuretools.PayloadTree(items)

    assert systemtools.TestManager.compare(
        tree,
        r'''
        datastructuretools.PayloadTree(
            [
                ['some', 'text'],
                ['more', 'text'],
                ]
            )
        '''
        )


# -*- encoding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import set_


def make_reference_manual_lilypond_file(music=None):
    r'''Makes reference manual LilyPond file.

        >>> score = Score([Staff('c d e f')])
        >>> lilypond_file = \
        ...     documentationtools.make_reference_manual_lilypond_file(score)

    ..  doctest::

        >>> print(format(lilypond_file)) # doctest: +SKIP

        \version "2.15.37"
        \language "english"

        \layout {
            indent = #0
            ragged-right = ##t
            \context {
                \Score
                \remove Bar_number_engraver
                \override SpacingSpanner #'strict-grace-spacing = ##t
                \override SpacingSpanner #'strict-note-spacing = ##t
                \override SpacingSpanner #'uniform-stretching = ##t
                \override TupletBracket #'bracket-visibility = ##t
                \override TupletBracket #'minimum-length = #3
                \override TupletBracket #'padding = #2
                \override TupletBracket #'springs-and-rods = #ly:spanner::set-spacing-rods
                \override TupletNumber #'text = #tuplet-number::calc-fraction-text
                proportionalNotationDuration = #(ly:make-moment 1 32)
                tupletFullLength = True
            }
        }

        \paper {
            left-margin = 1.0\in
        }

        \score {
            \new Score <<
                \new Staff {
                    c4
                    d4
                    e4
                    f4
                }
            >>
        }

    Returns LilyPond file.
    '''
    from abjad.tools import lilypondfiletools
    from abjad.tools import schemetools

    assert '__illustrate__' in dir(music)
    lilypond_file = music.__illustrate__()

    # header
    lilypond_file.header_block.tagline = markuptools.Markup('""')

    # layout
    lilypond_file.layout_block.indent = 0
    lilypond_file.line_width = lilypondfiletools.LilyPondDimension(6, 'in')
    lilypond_file.layout_block.ragged_right = True

    # paper
    lilypond_file.paper_block.left_margin = \
        lilypondfiletools.LilyPondDimension(1, 'in')

    # score context
    context_block = lilypondfiletools.ContextBlock(
        source_context_name='Score',
        )
    context_block.remove_commands.append('Bar_number_engraver')
    override(context_block).spacing_spanner.strict_grace_spacing = True
    override(context_block).spacing_spanner.strict_note_spacing = True
    override(context_block).spacing_spanner.uniform_stretching = True
    override(context_block).tuplet_bracket.bracket_visibility = True
    override(context_block).tuplet_bracket.padding = 2
    scheme = schemetools.Scheme('ly:spanner::set-spacing-rods')
    override(context_block).tuplet_bracket.springs_and_rods = scheme
    override(context_block).tuplet_bracket.minimum_length = 3
    scheme = schemetools.Scheme('tuplet-number::calc-fraction-text')
    override(context_block).tuplet_number.text = scheme
    moment = schemetools.SchemeMoment((1, 32))
    set_(context_block).proportionalNotationDuration = moment
    set_(context_block).tupletFullLength = True
    lilypond_file.layout_block.items.append(context_block)

    # etc
    lilypond_file.file_initial_system_comments[:] = []
    lilypond_file.global_staff_size = 12

    return lilypond_file



# -*- encoding: utf-8 -*-
import copy
from abjad.tools.datastructuretools.TypedList import TypedList


class OctaveTranspositionMapping(TypedList):
    '''An octave transposition mapping.

    ::

        >>> mapping = pitchtools.OctaveTranspositionMapping(
        ...     [('[A0, C4)', 15), ('[C4, C8)', 27)])

    ::

        >>> mapping
        OctaveTranspositionMapping([('[A0, C4)', 15), ('[C4, C8)', 27)])

    Octave transposition mappings model
    ``pitchtools.transpose_pitch_number_by_octave_transposition_mapping``
    input.

    Octave transposition mappings implement the list interface and
    are mutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### SPECIAL METHODS ###

    def __call__(self, pitches):
        r'''Call octave transposition mapping on `pitches`.

            >>> mapping([-24, -22, -23, -21])
            [24, 26, 25, 15]

        ::

            >>> mapping([0, 2, 1, 3])
            [36, 38, 37, 27]

        Returns list.
        '''
        transposed_pitches = [self._transpose_pitch(x) for x in pitches]
        return transposed_pitches

    def __format__(self, format_specification=''):
        r'''Formats octave transposition mapping.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ::

            >>> print(format(mapping))
            pitchtools.OctaveTranspositionMapping(
                [
                    pitchtools.OctaveTranspositionMappingComponent(
                        source_pitch_range=pitchtools.PitchRange(
                            range_string='[A0, C4)',
                            ),
                        target_octave_start_pitch=pitchtools.NumberedPitch(15),
                        ),
                    pitchtools.OctaveTranspositionMappingComponent(
                        source_pitch_range=pitchtools.PitchRange(
                            range_string='[C4, C8)',
                            ),
                        target_octave_start_pitch=pitchtools.NumberedPitch(27),
                        ),
                    ]
                )

        Returns string.
        '''
        superclass = super(OctaveTranspositionMapping, self)
        return superclass.__format__(format_specification=format_specification)

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import systemtools
        return systemtools.AttributeManifest()

    @staticmethod
    def _item_callable(expr):
        from abjad.tools import pitchtools
        if isinstance(expr, tuple):
            component = pitchtools.OctaveTranspositionMappingComponent(*expr)
        elif isinstance(expr, pitchtools.OctaveTranspositionMappingComponent):
            component = copy.copy(expr)
        else:
            raise TypeError(repr(expr))
        return component

    @property
    def _one_line_menu_summary(self):
        name = 'mapping'
        contents = []
        for mapping_component in self:
            contents.append(mapping_component._one_line_menu_summary)
        contents_string = ', '.join(contents)
        return '{}: {}'.format(name, contents_string)

    @property
    def _repr_specification(self):
        from abjad.tools import systemtools
        input_argument_tokens = []
        for mapping_component in self:
            item = (
                mapping_component.source_pitch_range.one_line_named_pitch_repr,
                mapping_component.target_octave_start_pitch.pitch_number
                )
            input_argument_tokens.append(item)
        keyword_argument_names = []
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            keyword_argument_names=keyword_argument_names,
            positional_argument_values=(
                input_argument_tokens,
                ),
            )

    ### PRIVATE METHODS ###

    def _transpose_pitch(self, pitch):
        from abjad.tools import pitchtools
        pitch = pitchtools.NamedPitch(pitch)
        target_pitch_class_number = pitch.pitch_class_number
        for component in self:
            if pitch in component.source_pitch_range:
                target_octave = range(
                    component.target_octave_start_pitch.pitch_number,
                    component.target_octave_start_pitch.pitch_number
                    + 12)
                for candidate_pitch in target_octave:
                    if candidate_pitch % 12 == \
                        pitch.pitch_class_number:
                        return candidate_pitch
        return pitch


# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_mathtools_partition_integer_into_halves_01():
    assert mathtools.partition_integer_into_halves(7, bigger='left') == (4, 3)
    assert mathtools.partition_integer_into_halves(7, bigger='right') == (3, 4)


def test_mathtools_partition_integer_into_halves_02():
    assert mathtools.partition_integer_into_halves(8, bigger='left') == (4, 4)
    assert mathtools.partition_integer_into_halves(8, bigger='right') == (4, 4)
    assert mathtools.partition_integer_into_halves(8, bigger='left', even='disallowed') == (5, 3)
    assert mathtools.partition_integer_into_halves(8, bigger='right', even='disallowed') == (3, 5)


def test_mathtools_partition_integer_into_halves_03():
    r'''Partition zero into halves.
    '''

    assert mathtools.partition_integer_into_halves(0, bigger='left') == (0, 0)
    assert mathtools.partition_integer_into_halves(0, bigger='right') == (0, 0)


def test_mathtools_partition_integer_into_halves_04():
    r'''Divide zero only into even halves.
    '''

    statement = "mathtools.partition_integer_into_halves(0, even='disallowed')"
    assert pytest.raises(PartitionError, statement)


def test_mathtools_partition_integer_into_halves_05():
    r'''Raise TypeError on noninteger n.
    Raise ValueError on negative n.
    '''

    statement = "mathtools.partition_integer_into_halves('foo')"
    assert pytest.raises(TypeError, statement)
    statement = 'mathtools.partition_integer_into_halves(-1)'
    assert pytest.raises(ValueError, statement)


# -*- encoding: utf-8 -*-
from abjad import *


def test_markuptools_Markup___format___01():

    markup = markuptools.Markup(r'\bold { foo }')

    assert systemtools.TestManager.compare(
        format(markup, 'storage'),
        r'''
        markuptools.Markup(
            contents=(
                markuptools.MarkupCommand(
                    'bold',
                    ['foo']
                    ),
                ),
            )
        '''
        )


def test_markuptools_Markup___format___02():

    markup = markuptools.Markup(
        r'\bold { allegro ma non troppo }',
        )

    assert systemtools.TestManager.compare(
        format(markup, 'storage'),
        r'''
        markuptools.Markup(
            contents=(
                markuptools.MarkupCommand(
                    'bold',
                    ['allegro', 'ma', 'non', 'troppo']
                    ),
                ),
            )
        '''
        )


# -*- encoding: utf-8 -*-


def combine_markup_commands(*commands):
    r'''Combine MarkupCommand and/or string objects.

    LilyPond's '\combine' markup command can only take two arguments, so in order
    to combine more than two stencils, a cascade of '\combine' commands must be
    employed.  `combine_markup_commands` simplifies this process.

    ::

        >>> markup_a = markuptools.MarkupCommand('draw-circle', 4, 0.4, False)
        >>> markup_b = markuptools.MarkupCommand(
        ...     'filled-box',
        ...     schemetools.SchemePair(-4, 4),
        ...     schemetools.SchemePair(-0.5, 0.5), 1)
        >>> markup_c = "some text"

    ::

        >>> markup = markuptools.combine_markup_commands(markup_a, markup_b, markup_c)
        >>> result = format(markup, 'lilypond')

    ::

        >>> print(result)
        \combine \combine \draw-circle #4 #0.4 ##f
            \filled-box #'(-4 . 4) #'(-0.5 . 0.5) #1 "some text"

    Returns a markup command instance, or a string if that was the only argument.
    '''
    from abjad.tools import markuptools

    assert len(commands)
    assert all(isinstance(command, (markuptools.MarkupCommand, str)) for command in commands)

    if 1 == len(commands):
        return commands[0]

    combined = markuptools.MarkupCommand('combine', commands[0], commands[1])
    for command in commands[2:]:
        combined = markuptools.MarkupCommand('combine', combined, command)
    return combined



# -*- encoding: utf-8 -*-
import os
import subprocess


def graph(expr, image_format='pdf', layout='dot'):
    r'''Graphs `expr` with graphviz and opens resulting image in
    the default image viewer.

    ::

        >>> rtm_syntax = '(3 ((2 (2 1)) 2))'
        >>> rhythm_tree = rhythmtreetools.RhythmTreeParser()(rtm_syntax)[0]
        >>> print(rhythm_tree.pretty_rtm_format)
        (3 (
            (2 (
                2
                1))
            2))

    ::

        >>> topleveltools.graph(rhythm_tree) # doctest: +SKIP

    Returns none.
    '''
    from abjad import abjad_configuration
    from abjad.tools import systemtools

    assert image_format in ('pdf', 'png')
    layouts =('circo', 'dot', 'fdp', 'neato', 'osage', 'sfdp', 'twopi')
    assert layout in layouts
    message = 'cannot find `{}` command-line tool.'
    message = message.format(layout)
    message += ' Please download Graphviz from graphviz.org.'
    assert systemtools.IOManager.find_executable(layout), message

    if isinstance(expr, str):
        graphviz_format = expr
    else:
        graphviz_format = expr.graphviz_format

    current_directory = os.path.abspath('.')
    ABJADOUTPUT = abjad_configuration['abjad_output_directory']
    systemtools.IOManager._ensure_directory_existence(ABJADOUTPUT)
    dot_path = os.path.join(
        ABJADOUTPUT,
        systemtools.IOManager.get_next_output_file_name(file_extension='dot'),
        )
    img_path = os.path.join(ABJADOUTPUT, dot_path.replace('dot', 'pdf'))

    with open(dot_path, 'w') as f:
        f.write(graphviz_format)

    command = '{} -v -T{} {} -o {}'
    command = command.format(layout, image_format, dot_path, img_path)
    subprocess.call(command, shell=True)

    pdf_viewer = abjad_configuration['pdf_viewer']
    ABJADOUTPUT = abjad_configuration['abjad_output_directory']
    systemtools.IOManager.open_file(img_path, pdf_viewer)



# -*- encoding: utf-8 -*-
from abjad.tools import sievetools
from abjad.tools.sievetools.ResidueClass import ResidueClass
from abjad.tools.sievetools.Sieve import Sieve
import pytest


RC = ResidueClass

def test_sievetools_ResidueClass_equality_01():
    r'''non-equal residue classes
    '''

    t1 = RC(2, 1)
    t2 = RC(3, 1)

    assert t1 != t2


def test_sievetools_ResidueClass_equality_02():
    r'''non-equal objects
    '''

    rc = RC(2, 1)

    assert rc != 'a'
    assert 2 != rc


def test_sievetools_ResidueClass_equality_03():
    r'''equal
    '''

    t1 = RC(2, 1)
    t2 = RC(2, 1)

    assert t1 == t2


# -*- encoding: utf-8 -*-
from abjad import *


def test_spannertools_Spanner_extend_left_01():
    r'''Extend spanner to the left.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    beam = Beam()
    attach(beam, voice[1])
    beam._extend_left(voice[0][:])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
            {
                g'8
                a'8
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_spannertools_Spanner_extend_left_02():
    r'''Extend spanner to the left.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    beam = Beam()
    attach(beam, voice[1])
    beam._extend_left(voice[0:1])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
            {
                g'8
                a'8
            }
        }
        '''
        )


# -*- encoding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate


def label_vertical_moments_in_expr_with_interval_class_vectors(
    expr, markup_direction=Down):
    r'''Label interval-class vector of every vertical moment in `expr`:

    ::

        >>> score = Score([])
        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> score.append(staff)
        >>> staff = Staff(r"""\clef "alto" g4 f4""")
        >>> score.append(staff)
        >>> staff = Staff(r"""\clef "bass" c,2""")
        >>> score.append(staff)

    ::

        >>> labeltools.label_vertical_moments_in_expr_with_interval_class_vectors(score)

    ..  doctest::

        >>> print(format(score))
        \new Score <<
            \new Staff {
                c'8
                d'8
                    _ \markup {
                        \tiny
                            0010020
                        }
                e'8
                f'8
                    _ \markup {
                        \tiny
                            1000020
                        }
            }
            \new Staff {
                \clef "alto"
                g4
                f4
                    _ \markup {
                        \tiny
                            0100110
                        }
            }
            \new Staff {
                \clef "bass"
                c,2
                    _ \markup {
                        \tiny
                            1000020
                        }
            }
        >>

    ::

        >>> show(score) # doctest: +SKIP

    Returns none.
    '''

    for vertical_moment in \
        iterate(expr).by_vertical_moment():
        leaves = vertical_moment.leaves
        pitches = pitchtools.PitchSegment.from_selection(leaves)
        if not pitches:
            continue
        interval_class_vector = \
            pitchtools.numbered_inversion_equivalent_interval_class_dictionary(
            pitches)
        formatted = _format_interval_class_vector(interval_class_vector)
        markup = markuptools.Markup(formatted, markup_direction)
        attach(markup, vertical_moment.start_leaves[-1])


def _format_interval_class_vector(interval_class_vector):
    counts = []
    for i in range(7):
        counts.append(interval_class_vector[i])
    counts = ''.join([str(x) for x in counts])
    if len(interval_class_vector) == 13:
        quartertones = []
        for i in range(6):
            quartertones.append(interval_class_vector[i+0.5])
        quartertones = ''.join([str(x) for x in quartertones])
        return r'\tiny \column { "%s" "%s" }' % (counts, quartertones)
    else:
        return r'\tiny %s' % counts



# -*- encoding: utf-8 -*-
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools import mathtools
from abjad.tools import rhythmtreetools
from abjad.tools import scoretools
from abjad.tools import sequencetools
from abjad.tools.abctools import AbjadObject


class Meter(AbjadObject):
    '''A rhythm tree-based model of nested time signature groupings.

    The structure of the tree corresponds to the monotonically increasing
    sequence of factors of the time signature's numerator.

    Each deeper level of the tree divides the previous by the next factor in
    sequence.

    Prime divisions greater than ``3`` are converted to sequences of ``2`` and
    ``3`` summing to that prime.  Hence ``5`` becomes ``3+2`` and ``7`` becomes
    ``3+2+2``.

    The meter models many parts of the common practice understanding of meter:

    ::

        >>> meter = metertools.Meter((4, 4))

    ::

        >>> meter
        Meter('(4/4 (1/4 1/4 1/4 1/4))')

    ::

        >>> print(meter.pretty_rtm_format)
        (4/4 (
            1/4
            1/4
            1/4
            1/4))

    ::

        >>> meter = metertools.Meter((3, 4))
        >>> print(meter.pretty_rtm_format)
        (3/4 (
            1/4
            1/4
            1/4))

    ::

        >>> meter = metertools.Meter((6, 8))
        >>> print(meter.pretty_rtm_format)
        (6/8 (
            (3/8 (
                1/8
                1/8
                1/8))
            (3/8 (
                1/8
                1/8
                1/8))))

    ::

        >>> meter = metertools.Meter((7, 4))
        >>> print(meter.pretty_rtm_format)
        (7/4 (
            (3/4 (
                1/4
                1/4
                1/4))
            (2/4 (
                1/4
                1/4))
            (2/4 (
                1/4
                1/4))))

    ::

        >>> meter = metertools.Meter(
        ...     (7, 4), decrease_durations_monotonically=False)
        >>> print(meter.pretty_rtm_format)
        (7/4 (
            (2/4 (
                1/4
                1/4))
            (2/4 (
                1/4
                1/4))
            (3/4 (
                1/4
                1/4
                1/4))))

    ::

        >>> meter = metertools.Meter((12, 8))
        >>> print(meter.pretty_rtm_format)
        (12/8 (
            (3/8 (
                1/8
                1/8
                1/8))
            (3/8 (
                1/8
                1/8
                1/8))
            (3/8 (
                1/8
                1/8
                1/8))
            (3/8 (
                1/8
                1/8
                1/8))))

    Returns meter object.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_decrease_durations_monotonically',
        '_denominator',
        '_numerator',
        '_root_node',
        )

    ### INITIALIZER ###

    def __init__(self, arg=None, decrease_durations_monotonically=True):

        arg = arg or (4, 4)

        def recurse(
            node, factors, denominator, decrease_durations_monotonically):
            if factors:
                factor, factors = factors[0], factors[1:]
                preprolated_duration = node.preprolated_duration.__div__(factor)
                if factor in (2, 3, 4, 5):
                    if factors:
                        for _ in range(factor):
                            child = rhythmtreetools.RhythmTreeContainer(
                                preprolated_duration=preprolated_duration)
                            node.append(child)
                            recurse(
                                child,
                                factors,
                                denominator,
                                decrease_durations_monotonically,
                                )
                    else:
                        for _ in range(factor):
                            node.append(
                                rhythmtreetools.RhythmTreeLeaf(
                                    preprolated_duration=(1, denominator)))
                else:
                    parts = [3]
                    total = 3
                    while total < factor:
                        if decrease_durations_monotonically:
                            parts.append(2)
                        else:
                            parts.insert(0, 2)
                        total += 2
                    for part in parts:
                        grouping = rhythmtreetools.RhythmTreeContainer(
                            preprolated_duration=part * preprolated_duration)
                        if factors:
                            for _ in range(part):
                                child = rhythmtreetools.RhythmTreeContainer(
                                    preprolated_duration=preprolated_duration)
                                grouping.append(child)
                                recurse(
                                    child,
                                    factors,
                                    denominator,
                                    decrease_durations_monotonically,
                                    )
                        else:
                            for _ in range(part):
                                grouping.append(
                                    rhythmtreetools.RhythmTreeLeaf(
                                        preprolated_duration=(1, denominator)))
                        node.append(grouping)
            else:
                node.extend([rhythmtreetools.RhythmTreeLeaf(
                    preprolated_duration=(1, denominator))
                    for _ in range(node.preprolated_duration.numerator)])

        decrease_durations_monotonically = \
            bool(decrease_durations_monotonically)

        if isinstance(arg, type(self)):
            root = arg.root_node
            numerator, denominator = arg.numerator, arg.denominator
            decrease_durations_monotonically = \
                arg.decrease_durations_monotonically

        elif isinstance(arg, (str, rhythmtreetools.RhythmTreeContainer)):
            if isinstance(arg, str):
                parsed = rhythmtreetools.RhythmTreeParser()(arg)
                assert len(parsed) == 1
                root = parsed[0]
            else:
                root = arg
            for node in root.nodes:
                assert node.prolation == 1
            numerator = root.preprolated_duration.numerator
            denominator = root.preprolated_duration.denominator

        elif isinstance(arg, (tuple, scoretools.Measure)) or \
            (hasattr(arg, 'numerator') and hasattr(arg, 'denominator')):
            if isinstance(arg, tuple):
                fraction = mathtools.NonreducedFraction(arg)
            elif isinstance(arg, scoretools.Measure):
                time_signature = arg._get_effective(
                    indicatortools.TimeSignature)
                fraction = mathtools.NonreducedFraction(
                    time_signature.numerator, time_signature.denominator)
            else:
                fraction = mathtools.NonreducedFraction(
                    arg.numerator, arg.denominator)
            numerator, denominator = fraction.numerator, fraction.denominator
            factors = mathtools.factors(numerator)[1:]
            # group two nested levels of 2s into a 4
            if 1 < len(factors) and factors[0] == factors[1] == 2:
                factors[0:2] = [4]
            root = rhythmtreetools.RhythmTreeContainer(
                preprolated_duration=fraction)
            recurse(
                root,
                factors,
                denominator,
                decrease_durations_monotonically,
                )

        else:
            message = 'can not initialize {}: {!r}.'
            message = message.format(type(self).__name__, arg)
            raise ValueError(message)

        self._root_node = root
        self._numerator = numerator
        self._denominator = denominator
        self._decrease_durations_monotonically = \
            decrease_durations_monotonically

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true when `expr` is a meter with an rtm format equal to that of
        this meter. Otherwise false.

        Returns boolean.
        '''
        if type(self) == type(expr):
            if self.rtm_format == expr.rtm_format:
                return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats meter.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

            >>> meter = metertools.Meter((7, 4))
            >>> print(format(meter))
            metertools.Meter(
                '(7/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4)) (2/4 (1/4 1/4))))'
                )

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __hash__(self):
        r'''Hashes meter.
        '''
        return hash((type(self), self.rtm_format))

    def __iter__(self):
        r'''Iterates meter.

        ::

            >>> meter = metertools.Meter((5, 4))

        ::

            >>> for x in meter:
            ...    x
            ...
            (NonreducedFraction(0, 4), NonreducedFraction(1, 4))
            (NonreducedFraction(1, 4), NonreducedFraction(2, 4))
            (NonreducedFraction(2, 4), NonreducedFraction(3, 4))
            (NonreducedFraction(3, 4), NonreducedFraction(4, 4))
            (NonreducedFraction(4, 4), NonreducedFraction(5, 4))
            (NonreducedFraction(0, 4), NonreducedFraction(5, 4))

        Yields pairs.
        '''
        def recurse(node):
            result = []
            for child in node:
                if isinstance(child, rhythmtreetools.RhythmTreeLeaf):
                    result.append(child)
                else:
                    result.extend(recurse(child))
            result.append(node)
            return result
        result = recurse(self.root_node)
        for x in result:
            start_offset = mathtools.NonreducedFraction(x.start_offset
                ).with_denominator(self.denominator)
            stop_offset = mathtools.NonreducedFraction(x.stop_offset
                ).with_denominator(self.denominator)
            yield start_offset, stop_offset

    ### PRIVATE PROPERTIES ###

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=(),
            positional_argument_values=(
                self.rtm_format,
                ),
            )

    ### PUBLIC PROPERTIES ###

    @property
    def decrease_durations_monotonically(self):
        r'''True if the meter divides large primes into collections of ``2``
        and ``3`` that decrease monotonically.

        ..  container:: example

            **Example 1.** Metrical hiearchy with durations that increase
            monotonically:

            ::

                >>> meter = metertools.Meter(
                ...     (7, 4),
                ...     decrease_durations_monotonically=False,
                ...     )

            ::

                >>> meter.decrease_durations_monotonically
                False

            ::

                >>> print(meter.pretty_rtm_format)
                (7/4 (
                    (2/4 (
                        1/4
                        1/4))
                    (2/4 (
                        1/4
                        1/4))
                    (3/4 (
                        1/4
                        1/4
                        1/4))))

        ..  container:: example

            **Example 2.** Meter with durations that
            decrease monotonically:

            ::

                >>> meter = \
                ...     metertools.Meter((7, 4),
                ...     decrease_durations_monotonically=True)

            ::

                >>> meter.decrease_durations_monotonically
                True

            ::

                >>> print(meter.pretty_rtm_format)
                (7/4 (
                    (3/4 (
                        1/4
                        1/4
                        1/4))
                    (2/4 (
                        1/4
                        1/4))
                    (2/4 (
                        1/4
                        1/4))))

        Returns boolean.
        '''
        return self._decrease_durations_monotonically

    @property
    def denominator(self):
        r'''Beat hierarchy denominator:

        ::

            >>> meter.denominator
            4

        Returns positive integer.
        '''
        return self._denominator

    @property
    def depthwise_offset_inventory(self):
        r'''Depthwise inventory of offsets at each grouping level:

        ::

            >>> for depth, offsets in enumerate(
            ...     meter.depthwise_offset_inventory):
            ...     print(depth, offsets)
            0 (Offset(0, 1), Offset(7, 4))
            1 (Offset(0, 1), Offset(3, 4), Offset(5, 4), Offset(7, 4))
            2 (Offset(0, 1), Offset(1, 4), Offset(1, 2), Offset(3, 4), Offset(1, 1), Offset(5, 4), Offset(3, 2), Offset(7, 4))

        Returns dictionary.
        '''
        inventory = []
        for depth, nodes in sorted(
            self.root_node.depthwise_inventory.items()):
            offsets = []
            for node in nodes:
                offsets.append(durationtools.Offset(node.start_offset))
            offsets.append(
                durationtools.Offset(self.numerator, self.denominator))
            inventory.append(tuple(offsets))
        return tuple(inventory)

    @property
    def graphviz_format(self):
        r'''Graphviz format of hierarchy's root node:

        ::

            >>> print(meter.graphviz_format)
            digraph G {
                node_0 [label="7/4",
                    shape=triangle];
                node_1 [label="3/4",
                    shape=triangle];
                node_2 [label="1/4",
                    shape=box];
                node_3 [label="1/4",
                    shape=box];
                node_4 [label="1/4",
                    shape=box];
                node_5 [label="2/4",
                    shape=triangle];
                node_6 [label="1/4",
                    shape=box];
                node_7 [label="1/4",
                    shape=box];
                node_8 [label="2/4",
                    shape=triangle];
                node_9 [label="1/4",
                    shape=box];
                node_10 [label="1/4",
                    shape=box];
                node_0 -> node_1;
                node_0 -> node_5;
                node_0 -> node_8;
                node_1 -> node_2;
                node_1 -> node_3;
                node_1 -> node_4;
                node_5 -> node_6;
                node_5 -> node_7;
                node_8 -> node_10;
                node_8 -> node_9;
            }

        ::

            >>> topleveltools.graph(meter) # doctest: +SKIP

        Returns string.
        '''
        return self.root_node.graphviz_format

    @property
    def implied_time_signature(self):
        r'''Implied time signature:

        ::

            >>> metertools.Meter((4, 4)).implied_time_signature
            TimeSignature((4, 4))

        Returns TimeSignature object.
        '''
        return indicatortools.TimeSignature(
            self.root_node.preprolated_duration)

    @property
    def numerator(self):
        r'''Beat hierarchy numerator:

        ::

            >>> meter.numerator
            7

        Returns positive integer.
        '''
        return self._numerator

    @property
    def preprolated_duration(self):
        r'''Beat hierarchy preprolated_duration:

        ::

            >>> meter.preprolated_duration
            Duration(7, 4)

        Returns preprolated_duration.
        '''
        return durationtools.Duration(self.numerator, self.denominator)

    @property
    def pretty_rtm_format(self):
        r'''Beat hiearchy pretty RTM format:

        ::

            >>> print(meter.pretty_rtm_format)
            (7/4 (
                (3/4 (
                    1/4
                    1/4
                    1/4))
                (2/4 (
                    1/4
                    1/4))
                (2/4 (
                    1/4
                    1/4))))

        Returns string.
        '''
        return self.root_node.pretty_rtm_format

    @property
    def root_node(self):
        r'''Beat hiearchy root node:

        ::

            >>> meter.root_node
            RhythmTreeContainer(
                children=(
                    RhythmTreeContainer(
                        children=(
                            RhythmTreeLeaf(
                                preprolated_duration=Duration(1, 4),
                                is_pitched=True
                                ),
                            RhythmTreeLeaf(
                                preprolated_duration=Duration(1, 4),
                                is_pitched=True
                                ),
                            RhythmTreeLeaf(
                                preprolated_duration=Duration(1, 4),
                                is_pitched=True
                                ),
                            ),
                        preprolated_duration=NonreducedFraction(3, 4)
                        ),
                    RhythmTreeContainer(
                        children=(
                            RhythmTreeLeaf(
                                preprolated_duration=Duration(1, 4),
                                is_pitched=True
                                ),
                            RhythmTreeLeaf(
                                preprolated_duration=Duration(1, 4),
                                is_pitched=True
                                ),
                            ),
                        preprolated_duration=NonreducedFraction(2, 4)
                        ),
                    RhythmTreeContainer(
                        children=(
                            RhythmTreeLeaf(
                                preprolated_duration=Duration(1, 4),
                                is_pitched=True
                                ),
                            RhythmTreeLeaf(
                                preprolated_duration=Duration(1, 4),
                                is_pitched=True
                                ),
                            ),
                        preprolated_duration=NonreducedFraction(2, 4)
                        ),
                    ),
                preprolated_duration=NonreducedFraction(7, 4)
                )

        Returns rhythm tree node.
        '''
        return self._root_node

    @property
    def rtm_format(self):
        r'''Beat hierarchy RTM format:

        ::

            >>> meter.rtm_format
            '(7/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4)) (2/4 (1/4 1/4))))'

        Returns string.
        '''
        return self._root_node.rtm_format

    ### PRIVATE METHODS ###

    def _get_recurser(self):
        return recurse

    @staticmethod
    def _make_gridded_test_rhythm(grid_length, rhythm_number, denominator=16):
        r'''Make test rhythm number `rhythm_number` that fits `grid_length`.

        Returns selection of one or more possibly tied notes.

        ..  container:: example

            **Example 1.** The eight test rhythms that fit a length-``4``
            grid:

            ::

                >>> from abjad.tools.metertools import Meter
                >>> for rhythm_number in range(8):
                ...     notes = Meter._make_gridded_test_rhythm(
                ...         4, rhythm_number, denominator=4)
                ...     measure = Measure((4, 4), notes)
                ...     print('{}\t{}'.format(rhythm_number, str(measure)))
                ... 
                0	Measure((4, 4), "c'1")
                1	Measure((4, 4), "c'2. c'4")
                2	Measure((4, 4), "c'2 c'4 c'4")
                3	Measure((4, 4), "c'2 c'2")
                4	Measure((4, 4), "c'4 c'4 c'2")
                5	Measure((4, 4), "c'4 c'4 c'4 c'4")
                6	Measure((4, 4), "c'4 c'2 c'4")
                7	Measure((4, 4), "c'4 c'2.")

        ..  container:: example

            **Example 2.** The sixteenth test rhythms for that a length-``5``
            grid:

            ::

                >>> for rhythm_number in range(16):
                ...     notes = Meter._make_gridded_test_rhythm(
                ...         5, rhythm_number, denominator=4)
                ...     measure = Measure((5, 4), notes)
                ...     print('{}\t{}'.format(rhythm_number, str(measure)))
                ... 
                0	Measure((5, 4), "c'1 ~ c'4")
                1	Measure((5, 4), "c'1 c'4")
                2	Measure((5, 4), "c'2. c'4 c'4")
                3	Measure((5, 4), "c'2. c'2")
                4	Measure((5, 4), "c'2 c'4 c'2")
                5	Measure((5, 4), "c'2 c'4 c'4 c'4")
                6	Measure((5, 4), "c'2 c'2 c'4")
                7	Measure((5, 4), "c'2 c'2.")
                8	Measure((5, 4), "c'4 c'4 c'2.")
                9	Measure((5, 4), "c'4 c'4 c'2 c'4")
                10	Measure((5, 4), "c'4 c'4 c'4 c'4 c'4")
                11	Measure((5, 4), "c'4 c'4 c'4 c'2")
                12	Measure((5, 4), "c'4 c'2 c'2")
                13	Measure((5, 4), "c'4 c'2 c'4 c'4")
                14	Measure((5, 4), "c'4 c'2. c'4")
                15	Measure((5, 4), "c'4 c'1")

        Use for testing meter establishment.
        '''
        from abjad.tools import scoretools
        # check input
        assert mathtools.is_positive_integer(grid_length)
        assert isinstance(rhythm_number, int)
        assert mathtools.is_positive_integer_power_of_two(denominator)
        # find count of all rhythms that fit grid length
        rhythm_count = 2 ** (grid_length - 1)
        # read rhythm number cyclically to allow large and
        # negative rhythm numbers
        rhythm_number = rhythm_number % rhythm_count
        # find binary representation of rhythm
        binary_representation = \
            mathtools.integer_to_binary_string(rhythm_number)
        binary_representation = binary_representation.zfill(grid_length)
        # partition binary representation of rhythm
        parts = sequencetools.partition_sequence_by_value_of_elements(
            binary_representation)
        # find durations
        durations = [
            durationtools.Duration(len(part), denominator)
            for part in parts
            ]
        # make notes
        notes = scoretools.make_notes([0], durations)
        # return notes
        return notes

    ### PUBLIC METHODS ###

    @staticmethod
    def fit_meters_to_expr(
        expr,
        meters,
        denominator=32,
        discard_final_orphan_downbeat=True,
        maximum_repetitions=None,
        starting_offset=None,
        ):
        r'''Find the best-matching sequence of meters for the offsets
        contained in `expr`.

        ::

            >>> meters = [metertools.Meter(x)
            ...     for x in [(3, 4), (4, 4), (5, 4)]
            ...     ]

        ..  container:: example

            **Example 1.** Matching a series of hypothetical 4/4 measures:

            ::

                >>> expr = [(0, 4), (4, 4), (8, 4), (12, 4), (16, 4)]
                >>> for x in metertools.Meter.fit_meters_to_expr(
                ...     expr, meters):
                ...     print(x.implied_time_signature)
                ... 
                4/4
                4/4
                4/4
                4/4

        ..  container:: example

            **Example 2.** Matching a series of hypothetical 5/4 measures:

            ::

                >>> expr = [(0, 4), (3, 4), (5, 4), (10, 4), (15, 4), (20, 4)]
                >>> for x in metertools.Meter.fit_meters_to_expr(
                ...     expr, meters):
                ...     print(x.implied_time_signature)
                ... 
                3/4
                3/4
                4/4
                5/4
                5/4

        Offsets are coerced from `expr` via
        `MetricAccentKernel.count_offsets_in_expr()`.

        MetricalHierarchies are coerced from `meters` via
        `MetricalHierarchyInventory`.

        Returns list.
        '''
        from abjad.tools import metertools
        offset_counter = \
            metertools.MetricAccentKernel.count_offsets_in_expr(expr)
        ordered_offsets = sorted(offset_counter, reverse=True)
        if not ordered_offsets:
            return []
        if starting_offset is None:
            start_offset = durationtools.Offset(0)
        else:
            start_offset = durationtools.Offset(starting_offset)
        meter_inventory = datastructuretools.TypedTuple(
            items=meters,
            item_class=metertools.Meter,
            )
        longest_hierarchy = sorted(meter_inventory,
            key=lambda x: x.preprolated_duration, reverse=True)[0]
        longest_kernel_duration = max(
            x.preprolated_duration for x in meter_inventory)
        kernels = [x.generate_offset_kernel_to_denominator(denominator)
            for x in meter_inventory]
        current_start_offset = start_offset
        selected_hierarchies = []
        while len(ordered_offsets) and \
            ordered_offsets[-1] < current_start_offset:
            ordered_offsets.pop()
        while len(ordered_offsets) and \
            current_start_offset <= ordered_offsets[-1]:
            if len(ordered_offsets) == 1:
                if discard_final_orphan_downbeat:
                    if ordered_offsets[0] == current_start_offset:
                        break
            current_stop_offset = \
                current_start_offset + longest_kernel_duration
            current_offset_counter = {}
            for offset in reversed(ordered_offsets):
                if current_start_offset <= offset <= current_stop_offset:
                    current_offset_counter[offset - current_start_offset] = \
                        offset_counter[offset]
                else:
                    break
            if not current_offset_counter:
                winner = longest_hierarchy
            else:
                candidates = []
                if maximum_repetitions is not None:
                    fencepost = -1 * abs(maximum_repetitions)
                    meter_buffer = selected_hierarchies[fencepost:]
                for meter_index, kernel in enumerate(kernels):
                    if maximum_repetitions is not None and \
                        len(meter_buffer) == maximum_repetitions:
                        meter_set = set(meter_buffer +
                            [meter_inventory[meter_index]])
                        if len(meter_set) == 1 and 1 < len(meter_inventory):
                            continue
                    response = kernel(current_offset_counter)
                    candidates.append((response, meter_index))
                candidates.sort(key=lambda x: x[0], reverse=True)
                response, index = candidates[0]
                winner = meter_inventory[index]
            selected_hierarchies.append(winner)
            current_start_offset += winner.preprolated_duration
            while len(ordered_offsets) \
                and ordered_offsets[-1] < current_start_offset:
                ordered_offsets.pop()
        return selected_hierarchies

    def generate_offset_kernel_to_denominator(
        self,
        denominator,
        normalize=True,
        ):
        r'''Generate a dictionary of all offsets in a meter up
        to `denominator`, where the keys are the offsets and the values
        are the normalized weights of those offsets:

        ::

            >>> meter = \
            ...     metertools.Meter((4, 4))
            >>> kernel = \
            ...     meter.generate_offset_kernel_to_denominator(8)
            >>> for offset, weight in sorted(kernel.kernel.items()):
            ...     print('{!s}\t{!s}'.format(offset, weight))
            ... 
            0       3/16
            1/8     1/16
            1/4     1/8
            3/8     1/16
            1/2     1/8
            5/8     1/16
            3/4     1/8
            7/8     1/16
            1       3/16

        This is useful for testing how strongly a collection of offsets
        responds to a given meter.

        Returns dictionary.
        '''
        from abjad.tools import metertools
        assert mathtools.is_positive_integer_power_of_two(
            denominator // self.denominator)

        inventory = list(self.depthwise_offset_inventory)
        old_flag_count = durationtools.Duration(1, self.denominator).flag_count
        new_flag_count = durationtools.Duration(1, denominator).flag_count
        extra_depth = new_flag_count - old_flag_count
        for _ in range(extra_depth):
            old_offsets = inventory[-1]
            new_offsets = []
            for first, second in \
                sequencetools.iterate_sequence_nwise(old_offsets):
                new_offsets.append(first)
                new_offsets.append((first + second) / 2)
            new_offsets.append(old_offsets[-1])
            inventory.append(tuple(new_offsets))

        total = 0
        kernel = {}
        for offsets in inventory:
            for offset in offsets:
                if offset not in kernel:
                    kernel[offset] = 0
                kernel[offset] += 1
                total += 1

        if normalize:
            for offset, response in kernel.items():
                kernel[offset] = durationtools.Multiplier(response, total)

        return metertools.MetricAccentKernel(kernel)
